import ast

from ..converter import VKScriptConverter

WHILE_TEMPLATE = "while(%(test)s){%(body)s};"
IF_TEMPLATE = "if(%(test)s){%(content)s}%(other)s;"


@VKScriptConverter.register(ast.While)
def while_handler(node: ast.While):
    converter = VKScriptConverter.get_current()
    if node.orelse:
        raise NotImplementedError("while...else not implemented.")
    test = converter.convert_node(node.test)
    body = converter.convert_block(node.body)
    return WHILE_TEMPLATE % {"test": test, "body": body}


@VKScriptConverter.register(ast.If)
def if_handler(node: ast.If):
    converter = VKScriptConverter.get_current()
    test = converter.convert_node(node.test)
    content = converter.convert_block(node.body)
    if node.orelse:
        other = f"else{{{converter.convert_block(node.orelse)}}}"
    else:
        other = ""
    return IF_TEMPLATE % {"test": test, "content": content, "other": other}



import typing

from .base import BaseModel

# https://vk.com/dev/bots_docs?f=2.3


class ClientInfo(BaseModel):
    button_actions: typing.List[str] = None
    keyboard: bool = None
    inline_keyboard: bool = None
    lang_id: typing.Union[int, float] = None



import ast

from ..converter import VKScriptConverter


@VKScriptConverter.register(ast.Assign)
def assign_handler(node: ast.Assign):
    converter = VKScriptConverter.get_current()
    left = node.targets
    left_ = []
    for target in left:
        if target.__class__ == ast.Name:
            left_.append(target.id)
            converter.scope.locals.append(target.id)
        elif target.__class__ == ast.Subscript:
            pass
        elif target.__class__ == ast.Tuple:
            raise NotImplementedError("Tuple assignments are not allowed")
        else:
            raise NotImplementedError(
                f"Assignments of {target.__class__} are not implemented"
            )

    right = converter.convert_node(node.value)
    return "var " + ",".join(f"{target}={right}" for target in left_) + ";"


@VKScriptConverter.register(ast.AugAssign)
def aug_assign_handler(node: ast.AugAssign):
    converter = VKScriptConverter.get_current()
    ops = {
        ast.Add: "+",
        ast.Sub: "-",
        ast.Mult: "*",
        ast.Div: "/",
        ast.Pow: "**",
        ast.RShift: ">>",
        ast.LShift: "<<",
        ast.BitOr: "|",
        ast.BitAnd: "&",
        ast.Mod: "%",
    }

    if node.op.__class__ not in ops:
        raise NotImplementedError(f"Operation {node.op} is not implemented.")

    if (
        node.target.__class__ == ast.Name
        and node.target.id not in converter.scope.locals
    ):
        raise NameError(f"name '{node.target.id}' is not defined")
    target = converter.convert_node(node.target)
    return f"{target}={target}{ops[node.op.__class__]}({converter.convert_node(node.value)});"


@VKScriptConverter.register(ast.Name)
def name_handler(node: ast.Name):
    converter = VKScriptConverter.get_current()
    if node.id in converter.scope.locals:
        return node.id
    if node.id not in converter.scope.globals:
        raise NameError(f"name '{node.id}' is not defined")
    if (
        type(converter.scope.globals[node.id])
        not in (str, int, tuple, dict, list)
        and converter.scope.globals[node.id] is not None
    ):
        raise NotImplementedError(
            f'type "{type(converter.scope.globals[node.id])}" not allowed inside VK Script'
        )
    return converter.convert_node(
        ast.parse(repr(converter.scope.globals[node.id]), mode="eval").body
    )



from ..base import BaseModel


class Sticker(BaseModel):
    id: int = None
    product_id: int = None
    photo_64: str = None
    photo_128: str = None
    photo_256: str = None
    photo_352: str = None
    width: int = None
    height: int = None



from ..base import BaseModel
from vk.types.attachments import Photo, Like

import typing


# https://vk.com/dev/objects/market_item


class MarketPriceCurrency(BaseModel):
    id: int = None
    name: str = None


class MarketPrice(BaseModel):
    amount: int = None
    currency: MarketPriceCurrency = None
    text: str = None


class MarketCategorySection(BaseModel):
    id: int = None
    name: str = None


class MarketCategory(BaseModel):
    id: int = None
    name: str = None
    section: MarketCategorySection = None


class Market(BaseModel):
    id: int = None
    owner_id: int = None
    title: str = None
    description: str = None
    price: MarketPrice = None
    category: MarketCategory = None
    thumb_photo: str = None
    date: int = None
    availability: int = None
    is_favorite: bool = None
    photos: typing.List[Photo] = None
    can_comment: int = None
    can_repost: int = None
    likes: Like = None
    url: str = None
    button_title: str = None



import typing

from vk.types.base import BaseModel


class CachedResponse(BaseModel):
    """
    Use this object to make a cached response
    """

    method_name: typing.AnyStr
    method_params: typing.Dict = {}



from typing import Union

from .base import BaseMethod
from vk.types.responses import market as m


class Market(BaseMethod):
    async def add(
        self,
        owner_id: int = None,
        name: str = None,
        description: str = None,
        category_id: int = None,
        price: Union[int, float] = None,
        old_price: Union[int, float] = None,
        deleted: bool = None,
        main_photo_id: int = None,
        photo_ids: list = None,
        url: str = None,
    ):
        """
        Add a new item to the market.
        :param owner_id: ID of an item owner community.
        :param name: Item name.
        :param description: Item description.
        :param category_id: Item category ID.
        :param price: Item price.
        :param old_price:
        :param deleted: Item status ('1' — deleted, '0' — not deleted).
        :param main_photo_id: Cover photo ID.
        :param photo_ids: IDs of additional photos.
        :param url: Url for button in market item.


        """
        method = self.get_method_name(self.add)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Add(**r)

    async def add_album(
        self,
        owner_id: int = None,
        title: str = None,
        photo_id: int = None,
        main_album: bool = None,
    ):
        """
        Create new collection of items
        :param owner_id: ID of an item owner community.
        :param title: Collection title.
        :param photo_id: Cover photo ID.
        :param main_album: Set as main ('1' – set, '0' – no).


        """
        method = self.get_method_name(self.add_album)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddAlbum(**r)

    async def add_to_album(
        self, owner_id: int = None, item_id: int = None, album_ids: list = None
    ):
        """
        Add an item to one or multiple collections.
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param album_ids: Collections IDs to add item to.


        """
        method = self.get_method_name(self.add_to_album)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddToAlbum(**r)

    async def create_comment(
        self,
        owner_id: int = None,
        item_id: int = None,
        message: str = None,
        attachments: list = None,
        from_group: bool = None,
        reply_to_comment: int = None,
        sticker_id: int = None,
        guid: str = None,
    ):
        """
        Create a new comment for an item.
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param message: Comment text (required if 'attachments' parameter is not specified)
        :param attachments: Comma-separated list of objects attached to a comment. The field is submitted the following way: , "'<owner_id>_<media_id>,<owner_id>_<media_id>'", , '' - media attachment type: "'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document", , '<owner_id>' - media owner id, '<media_id>' - media attachment id, , For example: "photo100172_166443618,photo66748_265827614",
        :param from_group: '1' - comment will be published on behalf of a community, '0' - on behalf of a user (by default).
        :param reply_to_comment: ID of a comment to reply with current comment to.
        :param sticker_id: Sticker ID.
        :param guid: Random value to avoid resending one comment.


        """
        method = self.get_method_name(self.create_comment)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.CreateComment(**r)

    async def delete(self, owner_id: int = None, item_id: int = None):
        """
        Delete an item.
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.


        """
        method = self.get_method_name(self.delete)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Delete(**r)

    async def delete_album(self, owner_id: int = None, album_id: int = None):
        """
        Delete a collection of items.
        :param owner_id: ID of an collection owner community.
        :param album_id: Collection ID.


        """
        method = self.get_method_name(self.delete_album)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.DeleteAlbum(**r)

    async def delete_comment(
        self, owner_id: int = None, comment_id: int = None
    ):
        """
        Delete an item's comment
        :param owner_id: identifier of an item owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param comment_id: comment id


        """
        method = self.get_method_name(self.delete_comment)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.DeleteComment(**r)

    async def edit(
        self,
        owner_id: int = None,
        item_id: int = None,
        name: str = None,
        description: str = None,
        category_id: int = None,
        price: Union[int, float] = None,
        deleted: bool = None,
        main_photo_id: int = None,
        photo_ids: list = None,
        url: str = None,
    ):
        """
        Edit an item.
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param name: Item name.
        :param description: Item description.
        :param category_id: Item category ID.
        :param price: Item price.
        :param deleted: Item status ('1' — deleted, '0' — not deleted).
        :param main_photo_id: Cover photo ID.
        :param photo_ids: IDs of additional photos.
        :param url: Url for button in market item.


        """
        method = self.get_method_name(self.edit)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Edit(**r)

    async def edit_album(
        self,
        owner_id: int = None,
        album_id: int = None,
        title: str = None,
        photo_id: int = None,
        main_album: bool = None,
    ):
        """
        Edit  a collection of items
        :param owner_id: ID of an collection owner community.
        :param album_id: Collection ID.
        :param title: Collection title.
        :param photo_id: Cover photo id
        :param main_album: Set as main ('1' – set, '0' – no).


        """
        method = self.get_method_name(self.edit_album)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.EditAlbum(**r)

    async def edit_comment(
        self,
        owner_id: int = None,
        comment_id: int = None,
        message: str = None,
        attachments: list = None,
    ):
        """
        Change item comment's text
        :param owner_id: ID of an item owner community.
        :param comment_id: Comment ID.
        :param message: New comment text (required if 'attachments' are not specified), , 2048 symbols maximum.
        :param attachments: Comma-separated list of objects attached to a comment. The field is submitted the following way: , "'<owner_id>_<media_id>,<owner_id>_<media_id>'", , '' - media attachment type: "'photo' - photo, 'video' - video, 'audio' - audio, 'doc' - document", , '<owner_id>' - media owner id, '<media_id>' - media attachment id, , For example: "photo100172_166443618,photo66748_265827614",


        """
        method = self.get_method_name(self.edit_comment)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.EditComment(**r)

    async def get(
        self,
        owner_id: int = None,
        album_id: int = None,
        count: int = None,
        offset: int = None,
        extended: bool = None,
    ):
        """
        Return items list for a community.
        :param owner_id: ID of an item owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param album_id:
        :param count: Number of items to return.
        :param offset: Offset needed to return a specific subset of results.
        :param extended: '1' – method will return additional fields: 'likes, can_comment, car_repost, photos'. These parameters are not returned by default.


        """
        method = self.get_method_name(self.get)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Get(**r)

    async def get_album_by_id(
        self, owner_id: int = None, album_ids: list = None
    ):
        """
        Return items album's data
        :param owner_id: identifier of an album owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param album_ids: collections identifiers to obtain data from


        """
        method = self.get_method_name(self.get_album_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetAlbumById(**r)

    async def get_albums(
        self, owner_id: int = None, offset: int = None, count: int = None
    ):
        """
        Return community's collections list.
        :param owner_id: ID of an items owner community.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of items to return.


        """
        method = self.get_method_name(self.get_albums)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetAlbums(**r)

    async def get_by_id(self, item_ids: list = None, extended: bool = None):
        """
        Return information about market items by their ids.
        :param item_ids: Comma-separated ids list: {user id}_{item id}. If an item belongs to a community -{community id} is used. " 'Videos' value example: , '-4363_136089719,13245770_137352259'"
        :param extended: '1' – to return additional fields: 'likes, can_comment, car_repost, photos'. By default: '0'.


        """
        method = self.get_method_name(self.get_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetById(**r)

    async def get_categories(self, count: int = None, offset: int = None):
        """
        Return a list of market categories.
        :param count: Number of results to return.
        :param offset: Offset needed to return a specific subset of results.


        """
        method = self.get_method_name(self.get_categories)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetCategories(**r)

    async def get_comments(
        self,
        owner_id: int = None,
        item_id: int = None,
        need_likes: bool = None,
        start_comment_id: int = None,
        offset: int = None,
        count: int = None,
        sort: str = None,
        extended: bool = None,
        fields: list = None,
    ):
        """
        Return comments list for an item.
        :param owner_id: ID of an item owner community
        :param item_id: Item ID.
        :param need_likes: '1' — to return likes info.
        :param start_comment_id: ID of a comment to start a list from (details below).
        :param offset:
        :param count: Number of results to return.
        :param sort: Sort order ('asc' — from old to new, 'desc' — from new to old)
        :param extended: '1' — comments will be returned as numbered objects, in addition lists of 'profiles' and 'groups' objects will be returned.
        :param fields: List of additional profile fields to return. See the [vk.com/dev/fields|details]


        """
        method = self.get_method_name(self.get_comments)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetComments(**r)

    async def remove_from_album(
        self, owner_id: int = None, item_id: int = None, album_ids: list = None
    ):
        """
        Remove an item from one or multiple collections.
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param album_ids: Collections IDs to remove item from.


        """
        method = self.get_method_name(self.remove_from_album)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemoveFromAlbum(**r)

    async def reorder_albums(
        self,
        owner_id: int = None,
        album_id: int = None,
        before: int = None,
        after: int = None,
    ):
        """
        Reorder the collections list.
        :param owner_id: ID of an item owner community.
        :param album_id: Collection ID.
        :param before: ID of a collection to place current collection before it.
        :param after: ID of a collection to place current collection after it.


        """
        method = self.get_method_name(self.reorder_albums)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.ReorderAlbums(**r)

    async def reorder_items(
        self,
        owner_id: int = None,
        album_id: int = None,
        item_id: int = None,
        before: int = None,
        after: int = None,
    ):
        """
        Change item place in a collection.
        :param owner_id: ID of an item owner community.
        :param album_id: ID of a collection to reorder items in. Set 0 to reorder full items list.
        :param item_id: Item ID.
        :param before: ID of an item to place current item before it.
        :param after: ID of an item to place current item after it.


        """
        method = self.get_method_name(self.reorder_items)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.ReorderItems(**r)

    async def report(
        self, owner_id: int = None, item_id: int = None, reason: int = None
    ):
        """
        Send a complaint to the item.
        :param owner_id: ID of an item owner community.
        :param item_id: Item ID.
        :param reason: Complaint reason. Possible values: *'0' — spam,, *'1' — child porn,, *'2' — extremism,, *'3' — violence,, *'4' — drugs propaganda,, *'5' — adult materials,, *'6' — insult.


        """
        method = self.get_method_name(self.report)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Report(**r)

    async def report_comment(
        self, owner_id: int = None, comment_id: int = None, reason: int = None
    ):
        """
        Send a complaint to the item's comment.
        :param owner_id: ID of an item owner community.
        :param comment_id: Comment ID.
        :param reason: Complaint reason. Possible values: *'0' — spam,, *'1' — child porn,, *'2' — extremism,, *'3' — violence,, *'4' — drugs propaganda,, *'5' — adult materials,, *'6' — insult.


        """
        method = self.get_method_name(self.report_comment)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.ReportComment(**r)

    async def restore(self, owner_id: int = None, item_id: int = None):
        """
        Restore recently deleted item
        :param owner_id: ID of an item owner community.
        :param item_id: Deleted item ID.


        """
        method = self.get_method_name(self.restore)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Restore(**r)

    async def restore_comment(
        self, owner_id: int = None, comment_id: int = None
    ):
        """
        Restore a recently deleted comment
        :param owner_id: identifier of an item owner community, "Note that community id in the 'owner_id' parameter should be negative number. For example 'owner_id'=-1 matches the [vk.com/apiclub|VK API] community "
        :param comment_id: deleted comment id


        """
        method = self.get_method_name(self.restore_comment)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RestoreComment(**r)

    async def search(
        self,
        owner_id: int = None,
        album_id: int = None,
        q: str = None,
        price_from: int = None,
        price_to: int = None,
        tags: list = None,
        sort: int = None,
        rev: int = None,
        offset: int = None,
        count: int = None,
        extended: bool = None,
        status: int = None,
    ):
        """
        Search market items in a community's catalog
        :param owner_id: ID of an items owner community.
        :param album_id:
        :param q: Search query, for example "pink slippers".
        :param price_from: Minimum item price value.
        :param price_to: Maximum item price value.
        :param tags: Comma-separated tag IDs list.
        :param sort:
        :param rev: '0' — do not use reverse order, '1' — use reverse order
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of items to return.
        :param extended: '1' – to return additional fields: 'likes, can_comment, car_repost, photos'. By default: '0'.
        :param status:


        """
        method = self.get_method_name(self.search)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Search(**r)



from .others import SimpleResponse
from ..base import BaseModel

from ..attachments.market import (
    Market,
    MarketCategorySection,
    MarketPrice,
    MarketCategory,
)
from ..attachments.market_album import MarketAlbum
from ..user import User
from ..attachments import Attachments
from ..attachments import Photo


import typing


class AddResponse(BaseModel):
    market_item_id: int = None


class Add(BaseModel):
    response: AddResponse = None


class AddAlbumResponse(BaseModel):
    market_album_id: int = None


class AddAlbum(BaseModel):
    response: AddAlbumResponse = None


class AddToAlbum(SimpleResponse):
    pass


class CreateComment(SimpleResponse):
    pass


class Delete(SimpleResponse):
    pass


class DeleteAlbum(SimpleResponse):
    pass


class DeleteComment(SimpleResponse):
    pass


class Edit(SimpleResponse):
    pass


class EditAlbum(SimpleResponse):
    pass


class EditComment(SimpleResponse):
    pass


class GetResponse(BaseModel):
    count: int = None
    items: typing.List[Market] = None


class Get(BaseModel):
    response: GetResponse = None


class GetAlbumByIdResponse(BaseModel):
    count: int = None
    items: typing.List[MarketAlbum] = None


class GetAlbumById(BaseModel):
    response: GetAlbumByIdResponse = None


class GetAlbums(BaseModel):
    response: GetAlbumByIdResponse = None


class GetById(BaseModel):
    response: GetResponse = None


class GetCategoriesResponseItem(BaseModel):
    id: int = None
    name: str = None
    section: MarketCategorySection = None


class GetCategoriesResponse(BaseModel):
    count: int = None
    items: typing.List[GetCategoriesResponseItem] = None


class GetCommentsResponse(BaseModel):
    count: int = None
    items: typing.List[typing.Dict] = None
    attachment: typing.List[Attachments] = None
    profiles: typing.List[User] = None


class GetComments(BaseModel):
    response: GetCommentsResponse = None


class RemoveFromAlbum(SimpleResponse):
    pass


class ReorderAlbums(SimpleResponse):
    pass


class ReorderItems(SimpleResponse):
    pass


class Report(SimpleResponse):
    pass


class ReportComment(SimpleResponse):
    pass


class Restore(SimpleResponse):
    pass


class RestoreComment(SimpleResponse):
    pass


class SearchResponse(BaseModel):
    count: int = None
    items: typing.List[typing.Any] = None
    date: int = None
    description: str = None
    external_id: str = None
    id: int = None
    owner_id: int = None
    price: MarketPrice = None
    thumb_photo: str = None
    title: str = None
    albums_ids: typing.List[int] = None
    photos: typing.List[Photo] = None
    can_comment: int = None
    can_repost: int = None
    likes: dict = None
    reposts: dict = None
    views_count: int = None


class Search(BaseModel):
    response: SearchResponse = None



from ..base import BaseModel
from enum import Enum

# Part of https://vk.com/dev/account.getProfileInfo?params[v]=5.101


class NameRequestStatus(Enum):
    processing = "processing"
    declined = "declined"


class NameRequest(BaseModel):
    id: int = None
    status: NameRequestStatus = None
    first_name: str = None
    last_name: str



from ..base import BaseModel

# https://vk.com/dev/objects/page


class Page(BaseModel):
    id: int = None
    group_id: int = None
    creator_id: int = None
    title: str = None
    current_user_can_edit: int = None
    current_user_can_edit_access: int = None
    who_can_view: int = None
    who_can_edit: int = None
    edited: int = None
    created: int = None
    editor_id: int = None
    views: int = None
    parent: str = None
    parent2: str = None
    source: str = None
    html: str = None
    view_url: str = None



"""
A part of library which represent a main object of VK API.
"""
import asyncio
import logging
import typing
from asyncio import AbstractEventLoop

try:
    from contextlib import asynccontextmanager
except ImportError:
    from async_generator import asynccontextmanager

from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientError

from vk.constants import API_LINK
from vk.constants import API_VERSION
from vk.constants import JSON_LIBRARY
from vk.exceptions import APIErrorDispatcher
from vk.methods import API
from vk.utils import ContextInstanceMixin

logger = logging.getLogger(__name__)

try:
    import uvloop  # noqa

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
except ImportError:
    uvloop = None

T = typing.TypeVar("T")


class HTTPException(ClientError):  # TODO: move to vk/exceptions
    pass


class VK(ContextInstanceMixin):
    """
    The main object of VKAPI, obtain basic methods to access the API.
    """

    def __init__(
        self,
        access_token: str,
        *,
        loop: AbstractEventLoop = None,
        client: ClientSession = None,
        change_vk_context_object=None,
    ):

        """

        :param str access_token: access token of VK user/community to access the VK methods.
        :param AbstractEventLoop loop: asyncio event loop, used in Task manager/dispatcher extensions/etc.
        :param ClientSession client: aiohttp client session.
        :param change_vk_context_object: change context of VK object.
        """
        self.access_token: str = access_token
        self.loop: asyncio.AbstractEventLoop = loop if loop is not None else asyncio.get_event_loop()
        self.client: ClientSession = (
            client
            if client is not None and isinstance(client, ClientSession)
            else ClientSession(json_serialize=JSON_LIBRARY.dumps)
        )

        self.error_dispatcher: APIErrorDispatcher = APIErrorDispatcher(self)
        self.__api_object = self.__get_api()
        if change_vk_context_object is None:
            self.set_current(self)

    async def _api_request(
        self,
        method_name: typing.AnyStr,
        params: dict = None,
        _raw_mode: bool = False,
        ignore_errors: bool = False,
    ) -> dict:
        """

        :param str method_name:
        :param dict params: parameters of method
        :param bool _raw_mode: signal to return 'raw' response, or not (basically, return response["response"])
        :param ignore_errors: signal to ignore errors
        :return:
        """
        if params:
            params = {k: v for k, v in params.items() if v is not None}

        elif params is None or not isinstance(params, dict):
            params = {}

        params.update({"v": API_VERSION, "access_token": self.access_token})
        logger.debug(f"Params to send: {params}")
        async with self.client.post(
            API_LINK + method_name, data=params
        ) as response:
            try:
                json: typing.Dict[str, typing.Any] = await response.json(
                    loads=JSON_LIBRARY.loads
                )
            except ClientError:
                logger.error("Some exception occured.. Can't load json.")
                text = await response.text()
                raise HTTPException(text)
            logger.debug(
                f"Method {method_name} called. Response from API: {json}"
            )
            if "error" in json:
                return await self.error_dispatcher.error_handle(
                    json, ignore_errors
                )

            if _raw_mode:
                return json

            return json["response"]

    async def api_request(
        self,
        method_name: str,
        params: dict = None,
        ignore_errors: bool = False,
    ) -> dict:
        """
        Send api request to the VK server
        :param method_name: method to execute
        :param params: parameters of method
        :param ignore_errors:
        :return:
        """
        return await self._api_request(
            method_name=method_name, params=params, ignore_errors=ignore_errors
        )

    async def execute_api_request(self, code: str) -> dict:
        """
        https://vk.com/dev/execute

        :param code: code to execute. Example: return API.status.get();
        :return:
        """
        return await self.api_request("execute", params={"code": code})

    def __get_api(self) -> API:
        """
        Get API class
        :return:
        """
        api = API(self)
        API.set_current(api)
        return api

    def get_api(self) -> API:
        """
        Get API class
        :return:
        """
        return self.__api_object

    @classmethod
    @asynccontextmanager
    async def with_token(cls: T, access_token: str) -> "VK":
        """
        Access VKAPI in one `async with` block.

        >>> vk: VK
        >>> async with VK.with_token("my_token") as vk:
        >>>     result = await vk.api_request("status.get")
        >>>     print(result)
        >>>     print(vk.client.closed)  # False
        >>> print(vk.client.closed)  # True
        :param access_token:
        :return:
        """
        vk = cls(access_token=access_token, change_vk_context_object=False)
        yield vk
        await vk.close()

    async def close(self) -> None:
        """
        Close aiohttp client session.
        :return:
        """
        if not self.client.closed:
            await self.client.close()

    def __del__(self):
        """
        Gonna call when python interpreter will try to free memory.
        :return:
        """
        if not self.loop.is_closed():
            self.loop.create_task(self.close())
            self.loop.close()



from ..base import BaseModel
from vk.types.additional import PhotoSizes
import typing


class PrettyCard(BaseModel):
    card_id: str = None
    link_url: str = None
    title: str = None
    images: typing.List[PhotoSizes] = None
    button: typing.Any = None
    price: str = None
    price_old: str = None


class PrettyCards(BaseModel):
    cards: typing.List[PrettyCard] = None



import logging

from ..dispatcher.extension import BaseExtension
from vk.constants import JSON_LIBRARY

logger = logging.getLogger(__name__)
try:
    import aio_pika
    from aio_pika.pool import Pool
except ImportError:
    aio_pika = None
    Pool = None


class RabbitMQ(BaseExtension):
    key = "rabbitmq"

    def __init__(
        self,
        vk,
        queue_name: str,
        rabbitmq_url: str = "amqp://guest:guest@127.0.0.1/",
        max_connections: int = 2,
        max_channels: int = 15,
    ):
        if aio_pika:
            self._vk = vk
            self._queue_name = queue_name
            self._url = rabbitmq_url
            self._conn_pool = Pool(
                self.get_connection, max_size=max_connections, loop=vk.loop
            )
            self._chann_pool = Pool(
                self.get_channel, max_size=max_channels, loop=vk.loop
            )
        else:
            raise RuntimeWarning(
                "Please install aio_pika (pip install aio_pika) for use this extension"
            )

    async def get_events(self) -> None:
        pass

    async def get_connection(self):
        return await aio_pika.connect_robust(self._url)

    async def get_channel(self) -> "aio_pika.Channel":
        async with self._conn_pool.acquire() as connection:
            return await connection.channel()

    async def run(self, dp):
        logger.info("RabbitMQ consumer started!")
        async with self._chann_pool.acquire() as channel:  # type: aio_pika.Channel
            await channel.set_qos(10)

            queue = await channel.declare_queue(
                self._queue_name, durable=False, auto_delete=False
            )

            async with queue.iterator() as queue_iter:
                async for message in queue_iter:
                    event = JSON_LIBRARY.loads(message.body.decode())
                    await dp._process_events([event])
                    await message.ack()



import typing
from enum import auto
from enum import Enum

import urllib3
from aiohttp import ClientSession

from vk.constants import API_VERSION
from vk.constants import JSON_LIBRARY
from vk.constants import JSONDecodeError
from vk.utils.mixins import ContextInstanceMixin


class ResponseErrorAction(Enum):
    NEED_VALIDATION = auto()


class ResponseValidation(typing.NamedTuple):
    status: bool
    action: ResponseErrorAction = None


class AppID(Enum):
    ANDORID = 2274003


class AppSecret(Enum):
    ANDROID = "hHbZxrka2uZ6jB1inYsH"


USER_AGENT = (
    "Mozilla/5.0 (Linux; Android 7.0; SM-G930V Build/NRD90M) AppleWebKit/537.36 "
    "(KHTML, like Gecko) Chrome/59.0.3071.125 Mobile Safari/537.36"
)
AUTH_URL = "https://oauth.vk.com/token"


class AuthManager(ContextInstanceMixin):
    def __init__(
        self,
        login: str,
        password: str,
        app_id: typing.Union[int, AppID] = AppID.ANDORID,
        client_secret: typing.Union[str, AppSecret] = AppSecret.ANDROID,
    ):
        self._client = urllib3.PoolManager(headers={"User-Agent": USER_AGENT})
        self._async_client = ClientSession(headers={"User-Agent": USER_AGENT})
        if isinstance(app_id, AppID):
            app_id = app_id.value
        if isinstance(client_secret, AppSecret):
            client_secret = client_secret.value
        self.app_id = app_id
        self.client_secret = client_secret
        self.login = login
        self.password = password

    def get_token(self):
        return self._auth()

    async def get_token_async(self):
        return await self._auth_async()

    def _auth(self) -> str:
        fields = {
            "grant_type": "client_credentials",
            "client_id": self.app_id,
            "client_secret": self.client_secret,
            "username": self.login,
            "password": self.password,
            "2fa_supported": 1,
            "v": API_VERSION,
        }
        json = self._make_request(fields)
        validate_info = self._validate_response(json)
        if validate_info.status:
            return json["access_token"]
        else:
            if validate_info.action is ResponseErrorAction.NEED_VALIDATION:
                while True:
                    result = self._process_need_validation(json)
                    if "access_token" not in result:
                        continue
                    return result["access_token"]

    async def _auth_async(self) -> str:
        fields = {
            "grant_type": "client_credentials",
            "client_id": self.app_id,
            "client_secret": self.client_secret,
            "username": self.login,
            "password": self.password,
            "2fa_supported": 1,
            "v": API_VERSION,
        }
        json = await self._make_request_async(fields)
        validate_info = self._validate_response(json)
        if validate_info.status:
            return json["access_token"]
        else:
            if validate_info.action is ResponseErrorAction.NEED_VALIDATION:
                while True:
                    result = await self._process_need_validation_async(json)
                    if "access_token" not in result:
                        continue
                    return result["access_token"]

    def _process_need_validation(self, json: dict):
        """
        :param json:
        :return:
        """
        self._make_request({}, url=json["redirect_uri"])
        code = self.error_need_validation()
        fields = {
            "grant_type": "client_credentials",
            "client_id": self.app_id,
            "client_secret": self.client_secret,
            "username": self.login,
            "password": self.password,
            "2fa_supported": 1,
            "v": API_VERSION,
            "code": code,
        }
        json = self._make_request(fields)
        return json

    async def _process_need_validation_async(self, json: dict):
        await self._make_request_async({}, url=json["redirect_uri"])
        code = self.error_need_validation()
        fields = {
            "grant_type": "client_credentials",
            "client_id": self.app_id,
            "client_secret": self.client_secret,
            "username": self.login,
            "password": self.password,
            "2fa_supported": 1,
            "v": API_VERSION,
            "code": code,
        }
        json = await self._make_request_async(fields)
        return json

    def _validate_response(self, response: dict):
        """
        Validate response
        :param response:
        :return:
        """
        if "error" in response:
            if response["error"] == "need_validation":
                return ResponseValidation(
                    False, ResponseErrorAction.NEED_VALIDATION
                )
            else:
                raise NotImplementedError()
        else:
            return ResponseValidation(True)

    def error_need_validation(self) -> int:  # noqa
        """
        Override this if needed.
        :return:
        """
        code = input("Enter the code which you will receive: ")
        return int(code.strip())

    async def _make_request_async(
        self, fields: dict, method: str = "GET", url: str = AUTH_URL
    ):
        async with self._async_client.request(
            method=method, url=url, params=fields
        ) as resp:
            try:
                json = await resp.json()
            except JSONDecodeError:
                json = {}
        return json

    async def close_async(self):
        await self._async_client.close()

    def _make_request(
        self, fields: dict, method: str = "GET", url: str = AUTH_URL
    ) -> dict:
        """
        Make request to API.
        :param fields:
        :param method:
        :param url:
        :return:
        """
        response: urllib3.HTTPResponse = self._client.request(
            method, url, fields
        )
        try:
            json = JSON_LIBRARY.loads(response.data.decode("utf-8"))
        # TODO: add universal exception for json validation error.
        except JSONDecodeError:
            json = {}
        return json



from .base import BaseModel
from .attachments import Photo

from enum import Enum, IntEnum

import typing


# https://vk.com/dev/objects/app


class ApplicationType(Enum):
    app = "social app"
    game = "game"
    site = "site"
    standalone = "standalone"


class LeaderBoardType(IntEnum):
    not_supported = 0
    by_level = 1
    by_points = 2


class Application(BaseModel):
    id: int = None
    title: str = None
    icon_278: str = None
    icon_139: str = None
    icon_150: str = None
    icon_75: str = None
    banner_560: str = None
    banner_1120: str = None
    type: ApplicationType = None
    section: str = None
    author_url: str = None
    author_id: int = None
    author_group: int = None
    members_count: int = None
    published_date: int = None
    catalog_position: int = None
    international: int = None
    leaderboard_type: LeaderBoardType = None
    genre_id: int = None
    genre: str = None
    platform_id: str = None
    is_in_catalog: int = None
    description: str = None
    screen_name: str = None
    icon_16: str = None
    screenshots: typing.List[Photo] = None



import asyncio
import logging
import typing

from vk.exceptions.errors import APIException

logger = logging.getLogger(__name__)


class APIErrorHandler:
    def __init__(self, error_code: int, handler: typing.Callable):
        self.handler: typing.Callable = handler
        self.error_code: int = error_code

    async def execute(self, error: typing.Dict):
        """
        Execute error handler
        :param error:
        :return:
        """
        try:
            return await self.handler(error)
        except Exception:  # noqa
            logging.exception("Exception occured in error handler...: ")


if typing.TYPE_CHECKING:
    from vk import VK


class APIErrorDispatcher:
    def __init__(self, vk: "VK"):
        """

        :param vk:
        """
        self.vk: "VK" = vk
        self._handlers: typing.List[APIErrorHandler] = []

        self._handlers.append(
            APIErrorHandler(6, self._to_many_requests_handler)
        )  # standard to many request handler

    async def _to_many_requests_handler(
        self, error: typing.Dict
    ) -> typing.Dict:
        logger.debug("To many requests error handle..")
        await asyncio.sleep(0.34)
        params = {}
        method_name = None
        for param in error["request_params"]:
            key = param["key"]
            value = param["value"]
            if key == "method":
                method_name = value
                continue

            params.update({key: value})

        return await self.vk.api_request(
            method_name=method_name, params=params
        )

    def error_handler(self, error_code: int):
        def decorator(coro: typing.Callable):
            self.register_error_handler(error_code, coro)

        return decorator

    def register_error_handler(self, error_code, coro):
        handler = APIErrorHandler(error_code, coro)
        self._handlers.append(handler)

    async def error_handle(
        self, json: typing.Dict, ignore_errors: bool = False
    ) -> typing.Union[typing.Dict, typing.NoReturn]:
        logger.debug("Some error from API handle..")
        error: dict = json["error"]
        logger.debug(f"Error data: {error}")

        code: int = error["error_code"]
        if not ignore_errors:
            for handler in self._handlers:
                if handler.error_code == code:
                    return await handler.execute(error)

        msg: str = error["error_msg"]
        raise APIException(code, msg)



from ..base import BaseModel

from vk.types.attachments import Photo


class Album(BaseModel):
    id: int = None
    thumb: Photo = None
    owner_id: int = None
    title: str = None
    description: str = None
    created: int = None
    updated: int = None
    size: int



from .others import SimpleResponse
from ..base import BaseModel

from ..additional import MetroStation


import typing


class GetChairsResponse(BaseModel):
    count: int = None
    items: typing.List[dict] = None


class GetChairs(BaseModel):
    response: GetChairsResponse = None


class GetCitiesResponse(BaseModel):
    count: int = None
    items: typing.List[dict]


class GetCities(BaseModel):
    response: GetCitiesResponse = None


class GetCitiesByIdResponse(BaseModel):
    id: int = None
    title: str = None


class GetCitiesById(BaseModel):
    response: typing.List[GetCitiesByIdResponse] = None


class GetCountriesResponse(BaseModel):
    count: int = None
    items: typing.List[GetCitiesByIdResponse] = None


class GetCountries(BaseModel):
    response: GetCountriesResponse = None


class GetCountriesById(BaseModel):
    response: typing.List[GetCitiesByIdResponse] = None


class GetFaculties(GetCountries):
    pass


class GetMetroStationsResponse(BaseModel):
    count: int = None
    items: typing.List[MetroStation] = None


class GetMetroStations(BaseModel):
    response: GetMetroStationsResponse = None


class GetMetroStationsById(BaseModel):
    response: typing.List[MetroStation] = None


class GetRegions(BaseModel):
    response: GetCountriesResponse = None


class GetSchoolClasses(BaseModel):
    response: typing.List = None


class GetSchools(BaseModel):
    response: typing.List[GetCountriesResponse] = None


class GetUniversities(BaseModel):
    response: typing.List[GetCountriesResponse] = None



from ..dispatcher.extension import BaseExtension
from vk.longpoll import BotLongPoll

import typing
import logging

logger = logging.getLogger(__name__)


class Polling(BaseExtension):
    key = "polling"

    def __init__(self, group_id: int, vk):
        self._longpoll: BotLongPoll = BotLongPoll(group_id, vk)

    async def get_events(self) -> typing.List:
        return await self._longpoll.listen()

    async def run(self, dp):
        await self._longpoll._prepare_longpoll()

        logger.info("Polling started!")

        while True:
            await dp._process_events(await self.get_events())



from ..base import BaseModel

import typing


class Place(BaseModel):
    id: int = None
    title: str = None
    latitude: typing.Union[int, float] = None
    longitude: typing.Union[int, float] = None
    type: str = None
    country: int = None
    city: int = None
    address: str = None


class GeoPlace(BaseModel):
    id: int = None
    title: str = None
    latitude: int = None
    longitude: int = None
    created: int = None
    icon: str = None
    country: str = None
    city: str = None
    type: int = None
    group_id: int = None
    group_photo: str = None
    checkins: int = None
    updated: int = None
    address: int = None



from .others import SimpleResponse
from ..base import BaseModel

from ..user import User

import typing


class Add(SimpleResponse):
    pass


class AddListResponse(BaseModel):
    list_id: int = None


class AddList(BaseModel):
    response: AddListResponse = None


class AreFriendsResponse(BaseModel):
    user_id: int = None
    friend_status: int = None
    sign: str = None


class AreFriends(BaseModel):
    response: typing.List[AreFriendsResponse] = None


class DeleteResponse(BaseModel):
    success: int = None
    friend_deleted: int = None
    out_request_deleted: int = None
    in_request_deleted: int = None
    suggestion_deleted: int = None


class Delete(BaseModel):
    response: DeleteResponse = None


class DeleteAllRequests(SimpleResponse):
    pass


class DeleteList(SimpleResponse):
    pass


class Edit(SimpleResponse):
    pass


class EditList(SimpleResponse):
    pass


class GetResponse(BaseModel):
    count: int = None
    items: typing.List[User] = None


class Get(BaseModel):
    response: GetResponse = None


class GetAppUsers(BaseModel):
    response: typing.List[int] = None


class GetByPhones(BaseModel):
    response: typing.List[User] = None


class GetListsResponseItem(BaseModel):
    name: str = None
    id: int = None


class GetListsResponse(BaseModel):
    count: int = None
    items: typing.List[GetListsResponseItem] = None


class GetLists(BaseModel):
    response: GetListsResponse = None


class GetMutual(BaseModel):
    response: typing.List[int] = None


class GetOnline(BaseModel):
    response: typing.List[int] = None


class GetRecent(BaseModel):
    response: typing.List[int] = None


class GetRequestsItem(User):
    mutual: typing.Any = None
    track_code: str = None


class GetRequestsResponse(BaseModel):
    count: int = None
    items: typing.List[GetRequestsItem] = None


class GetRequests(BaseModel):
    response: GetRequestsResponse = None


class GetSuggestionsResponse(BaseModel):
    count: int = None
    items: typing.List[User] = None


class GetSuggestions(BaseModel):
    response: GetSuggestionsResponse = None


class Search(BaseModel):
    response: GetSuggestionsResponse = None



import abc
import asyncio
import logging
import typing

from vk.bot_framework.dispatcher import data_
from vk.bot_framework.dispatcher.rule import BaseRule
from vk.types.events.community.events_list import Event
from vk.utils.mixins import MetaMixin

logger = logging.getLogger(__name__)


class BaseHandler(abc.ABC, MetaMixin):
    @property
    def event_type(self) -> Event:
        raise NotImplementedError()

    @property
    def handler(self) -> typing.Callable:
        raise NotImplementedError()

    @property
    def rules(self) -> typing.List[BaseRule]:
        raise NotImplementedError()

    @abc.abstractmethod
    async def execute_handler(self, *args) -> typing.Union[typing.Any, bool]:
        """
        Execute handler (after performing handler rules).
        args - (event, data)
        If return False - check next handlers.
        """


class SkipHandler(Exception):
    """
    Raise this if you want to skip handlers.
    """


class Handler(BaseHandler):
    def __init__(
        self,
        event_type: Event,
        handler: typing.Callable,
        rules: typing.List[BaseRule],
    ):
        """

        :param event_type: type of receiving event
        :param handler: coroutine
        :param rules: list of rules which is gonna be executed
        """
        self._event_type: Event = event_type
        self._handler: typing.Callable = handler
        self._rules: typing.List[BaseRule] = rules

    @property
    def event_type(self) -> Event:
        return self._event_type

    @property
    def handler(self) -> typing.Callable:
        return self._handler

    @property
    def rules(self) -> typing.List[BaseRule]:
        return self._rules

    async def execute_handler(self, *args):
        """
        Execute rules and handler
        :param args:
        :return:
        """
        # args - (event, data)
        if self.rules:
            _execute = False
            for rule in self.rules:
                if not asyncio.iscoroutinefunction(rule) and not isinstance(
                    rule, BaseRule
                ):
                    result = rule(*args)
                else:
                    result = await rule(*args)
                if not result:
                    _execute = False
                    return False
                if isinstance(result, dict):
                    args[1].update(result)
                    data_.set(args[1])
                _execute = True

            if _execute:
                return await self.handler(*args)
        else:
            return await self.handler(*args)



import typing
from enum import Enum

from .attachments import Attachment
from .attachments import Geo
from .base import BaseModel


# https://vk.com/dev/objects/message


class Action(Enum):
    chat_photo_update = "chat_photo_update"
    chat_photo_remove = "chat_photo_remove"
    chat_create = "chat_create"
    chat_title_update = "chat_title_update"
    chat_invite_user = "chat_invite_user"
    chat_kick_user = "chat_kick_user"
    chat_pin_message = "chat_pin_message"
    chat_unpin_message = "chat_unpin_message"
    chat_invite_user_by_link = "chat_invite_user_by_link"


class MessageActionPhoto(BaseModel):
    photo_50: str = None
    photo_100: str = None
    photo_200: str = None


class MessageAction(BaseModel):
    type: Action = None
    member_id: int = None
    text: str = None
    email: str = None
    photo: MessageActionPhoto = None


class Message(BaseModel):
    id: int = None
    date: int = None
    peer_id: int = None
    from_id: int = None
    text: str = None
    random_id: int = None
    attachments: typing.List[Attachment] = []
    important: bool = None
    geo: Geo = None
    payload: str = None
    action: MessageAction = None
    fwd_messages: typing.List["Message"] = []
    reply_message: "Message" = None
    is_cropped: bool = None

    @staticmethod
    def _separate_message(text: str) -> list:
        if not isinstance(text, bytes):
            in_bytes = text.encode("utf-8")
        else:
            in_bytes = text
        to_send: list = []
        if len(in_bytes) > 4096:
            chunks = [
                in_bytes[x : x + 4096] for x in range(0, len(in_bytes), 4096)
            ]
            [to_send.append(chunk.decode()) for chunk in chunks]

        return to_send

    async def _prepare_send(self, text: str, **kwargs):
        to_send = self._separate_message(text)
        sended = False
        if to_send:
            for s in to_send:
                sended = True
                await self.answer(message=s, **kwargs)
        return sended

    async def reply(
        self,
        message: str,
        attachment: str = None,
        keyboard: dict = None,
        **kwargs,
    ):
        """
        Answer to message with reply.
        :param message:
        :param attachment:
        :param keyboard:
        :return:
        """
        p = await self._prepare_send(
            message, attachment=attachment, keyboard=keyboard, **kwargs
        )
        if p:
            return

        return await self.api.messages.send(
            message=message,
            peer_id=self.peer_id,
            attachment=attachment,
            reply_to=self.id,
            keyboard=keyboard,
            random_id=0,
            **kwargs,
        )

    async def answer(
        self,
        message: str,
        attachment: str = None,
        keyboard: dict = None,
        **kwargs,
    ):
        """
        Answer to message without reply.
        :param message:
        :param attachment:
        :param keyboard:
        :return:
        """
        p = await self._prepare_send(
            message, attachment=attachment, keyboard=keyboard, **kwargs
        )
        if p:
            return

        return await self.api.messages.send(
            message=message,
            peer_id=self.peer_id,
            attachment=attachment,
            keyboard=keyboard,
            random_id=0,
            **kwargs,
        )

    async def cached_answer(
        self,
        message: str,
        attachment: str = None,
        keyboard: dict = None,
        **kwargs,
    ):
        from vk.bot_framework.addons.caching import CachedResponse

        """
        Answer to message without reply.
        :param message:
        :param attachment:
        :param keyboard:
        :return: cached object
        """
        await self.api.messages.send(
            message=message,
            peer_id=self.peer_id,
            attachment=attachment,
            keyboard=keyboard,
            random_id=0,
            **kwargs,
        )
        resp = CachedResponse(
            method_name="messages.send",
            method_params={
                "message": message,
                "attachment": attachment,
                "keyboard": keyboard,
                "random_id": 0,
                **kwargs,
            },
        )
        return resp

    def get_args(
        self, delete_element: int = 1, separator: str = None
    ) -> typing.List[str]:
        """
        Return message args splitted by whitespace without first (0) element.
        :param separator:
        :arg delete_element: return elemenets without 'delete_element - 1'
        :return:
        """
        try:
            return self.text.split(separator)[delete_element::]
        except IndexError:
            return []


Message.update_forward_refs()



"""
VK bot framework, easily to learn, fast to code, built on vk.py.
Designed specially for heavy loads.
"""
from .dispatcher import Dispatcher
from .dispatcher.dispatcher import get_group_id
from .dispatcher.handler import SkipHandler
from .dispatcher.storage import Storage
from .middlewares.middlewares import BaseMiddleware
from .rules import rules
from .rules.rules import BaseRule
from .rules.rules import NamedRule



from .others import SimpleResponse
from ..base import BaseModel


import typing


class CheckUserResponse(BaseModel):
    result: bool = None
    reason: str = None
    start_link: str = None
    sid: typing.Any = None


class CheckUser(BaseModel):
    response: CheckUserResponse = None


class CompleteResponse(BaseModel):
    limit: int = None
    day_limit: int = None
    spent: int = None
    cost: str = None
    test_mode: int = None
    success: int = None


class Complete(BaseModel):
    response: CompleteResponse = None


class GetStatsResponseDays(BaseModel):
    impressions: int = None
    started: int = None
    completed: int = None
    spent: int = None


class GetStatsResponse(BaseModel):
    limit: int = None
    spent: int = None
    cost: str = None
    impressions: int = None
    started: int = None
    completed: int = None
    days: typing.List[GetStatsResponseDays] = None


class GetStats(BaseModel):
    response: GetStatsResponse = None


class GetUsersResponse(BaseModel):
    uid: int = None
    aid: int = None
    sid: str = None
    date: int = None
    start_date: int = None
    status: int = None
    test_mode: int = None
    comment: str = None


class GetUsers(BaseModel):
    response: typing.List[GetUsersResponse] = None


class MetricHitResponse(BaseModel):
    result: bool = None
    redirect_link: str = None


class MetricHit(BaseModel):
    response: MetricHitResponse = None


class StartResponse(BaseModel):
    test_mode: int = None
    vk_sid: typing.Any = None


class Start(BaseModel):
    response: StartResponse = None



"""
A simple util for dispatcher for storage your data. e.g: database connection, messages count.
"""
import typing
from abc import ABC
from abc import abstractmethod


class AbstractStorage(ABC):
    @abstractmethod
    def place(self, key: typing.AnyStr, value: typing.Any) -> None:
        """
        Place value to storage.
        :param key:
        :param value:
        :return:
        """

    @abstractmethod
    def get(
        self, key: typing.AnyStr, default: typing.Any = None
    ) -> typing.Optional[typing.Any]:
        """
        Get value by the key from storage or get default value.
        :param key:
        :param default:
        :return:
        """

    @abstractmethod
    def delete(self, key: typing.AnyStr) -> None:
        """
        Delete key/value from storage by key
        :param key:
        :return:
        """

    @abstractmethod
    def update(self, key: typing.AnyStr, value: typing.Any):
        """
        Update value in storage by the key.
        :param key:
        :param value:
        :return:
        """

    @abstractmethod
    def exists(self, key: typing.AnyStr):
        """
        Check value exists in storage
        :param key:
        :return:
        """


class AbstractAsyncStorage(ABC):
    @abstractmethod
    async def place(self, key: typing.AnyStr, value: typing.Any) -> None:
        """
        Place value to storage.
        :param key:
        :param value:
        :return:
        """

    @abstractmethod
    async def get(
        self, key: typing.AnyStr, default: typing.Any = None
    ) -> typing.Optional[typing.Any]:
        """
        Get value by the key from storage or get default value.
        :param key:
        :param default:
        :return:
        """

    @abstractmethod
    async def delete(self, key: typing.AnyStr) -> None:
        """
        Delete key/value from storage by key
        :param key:
        :return:
        """

    @abstractmethod
    async def update(self, key: typing.AnyStr, value: typing.Any):
        """
        Update value in storage by the key.
        :param key:
        :param value:
        :return:
        """

    @abstractmethod
    async def exists(self, key: typing.AnyStr):
        """
        Check that value is exists in storage
        :param key:
        :return:
        """


class AbstractExpiredStorage(AbstractStorage):
    @abstractmethod
    def place(
        self, key: typing.AnyStr, value: typing.Any, expire: int
    ) -> None:  # noqa
        pass


class AbstractAsyncExpiredStorage(AbstractAsyncStorage):
    @abstractmethod
    async def place(
        self, key: typing.AnyStr, value: typing.Any, expire: int
    ) -> None:  # noqa
        pass


class Storage(AbstractStorage):
    """
    Basic storage
    """

    def __init__(self):
        self._dct = {}

    def place(self, key: typing.AnyStr, value: typing.Any) -> None:
        if key in self._dct:
            raise RuntimeError("Storage already have this key.")
        self._dct[key] = value

    def get(
        self, key: typing.AnyStr, default: typing.Any = None
    ) -> typing.Optional[typing.Any]:
        if key in self._dct:
            return self._dct[key]
        else:
            return default

    def delete(self, key: typing.AnyStr) -> None:
        if key in self._dct:
            del self._dct[key]
        else:
            raise RuntimeError("Undefined key.")

    def update(self, key: typing.AnyStr, value: typing.Any) -> None:
        if key not in self._dct:
            raise RuntimeError("Storage don`t have this key.")
        self._dct[key] = value

    def exists(self, key: typing.AnyStr):
        return key in self._dct



import logging
import typing

from vk import VK
from vk.bot_framework.dispatcher import data_
from vk.constants import default_extensions
from vk.constants import default_rules
from vk.exceptions.errors import APIException
from vk.types import BotEvent as Event
from vk.utils import ContextInstanceMixin
from vk.utils import time_logging
from vk.utils.deprecated import deprecated
from vk.utils.deprecated import deprecated_argument
from vk.utils.deprecated import warn_deprecated
from vk.utils.get_event import get_event_object
from .blueprints import Blueprint
from .extension import BaseExtension
from .extension import ExtensionsManager
from .handler import BaseHandler
from .handler import Handler
from .middleware import BaseMiddleware
from .middleware import MiddlewareManager
from .rule import BaseRule
from .rule import RuleFactory
from .storage import AbstractAsyncStorage
from .storage import AbstractStorage

logger = logging.getLogger(__name__)


async def get_group_id(vk: VK):
    try:
        result = await vk.api_request("groups.getById", ignore_errors=True)
        return result[0]["id"]
    except APIException:
        raise TypeError(f"group_id must be specified for user tokens")


class Dispatcher(ContextInstanceMixin):
    handler_class = Handler

    @deprecated_argument("group_id", "1.0.0")
    def __init__(self, vk: VK, group_id: int = None):
        if group_id:
            warn_deprecated(
                "Argument 'group_id' deprecated and removed in vk.py 1.0.0"
            )

        self._vk: VK = vk
        self._handlers: typing.List[BaseHandler] = []

        self._middleware_manager: MiddlewareManager = MiddlewareManager(self)
        self._rule_factory: RuleFactory = RuleFactory(default_rules())
        self._extensions_manager: ExtensionsManager = ExtensionsManager(
            self, default_extensions()
        )

        self._storage: typing.Optional[
            AbstractStorage, AbstractAsyncStorage
        ] = None

        self._registered_blueprints: typing.List[Blueprint] = []

        self.set_current(self)

    @property
    def handlers(self) -> typing.List[BaseHandler]:
        """
        Return a list of registered handlers.
        :return:
        """
        return self._handlers

    def get_handler(self, handler_coro: typing.Callable):
        """
        Get a handler object by the handler coroutine.
        :param handler_coro:
        :return:
        """
        for handler in self.handlers:
            if handler.handler is handler_coro:
                return handler

    @property
    def registered_blueprints(self) -> typing.List[Blueprint]:
        """
        Return a list of registered blueprints.
        :return:
        """
        return self._registered_blueprints

    @property
    @deprecated("Argument `group_id` deprecated")
    def group_id(self):
        return

    @property
    def vk(self):
        """
        Return a passed vk object.
        :return:
        """
        return self._vk

    @property
    def middlewares(self):
        """
        Return a list of registered middlewares.
        :return:
        """
        return self._middleware_manager.middlewares

    @property
    def storage(self):
        if not self._storage:
            raise RuntimeError("Storage not setuped.")
        return self._storage

    @storage.setter
    def storage(
        self, storage: typing.Union[AbstractAsyncStorage, AbstractStorage]
    ):
        """
        Set storage in dispatcher.
        :param storage:
        :return:
        """
        if not isinstance(storage, (AbstractStorage, AbstractAsyncStorage)):
            raise RuntimeError("Unexpected storage.")
        self._storage = storage

    def described_handler(
        self,
        name: str = None,
        description: str = None,
        deprecated: bool = False,
        **other_meta: dict,
    ):
        def decorator(coro: typing.Callable):
            handler = self.get_handler(coro)
            if not handler:
                raise RuntimeError("Handler not registered.")
            meta = {
                "name": name,
                "description": description,
                "deprecated": deprecated,
                **other_meta,
            }
            if handler.handler.__doc__:  # or set description in docstring
                meta["description"] = handler.handler.__doc__.strip()
            handler.meta = {k: v for k, v in meta.items() if v is not None}

        return decorator

    def _register_handler(self, handler: BaseHandler):
        """
        Append handler to the handlers list
        :param handler:
        :return:
        """
        self._handlers.append(handler)
        logger.debug(
            f"Handler '{handler.handler.__name__}' successfully added!"
        )

    def register_message_handler(
        self, coro: typing.Callable, rules: typing.List
    ):
        """
        Register message handler

        >>> dp.register_message_handler(my_handler, [])

        :param coro:
        :param rules:
        :return:
        """
        event_type = Event.MESSAGE_NEW
        handler = self.handler_class(event_type, coro, rules)
        self._register_handler(handler)

    def message_handler(
        self,
        *rules: typing.Tuple[
            typing.Type[BaseRule], typing.Callable, typing.Awaitable
        ],
        **named_rules: typing.Dict[str, typing.Any],
    ):
        """
        Register message handler with a decorator.

        :param rules: other user rules
        :param named_rules: other user named rules
        :return:
        """

        def decorator(coro: typing.Callable):
            nonlocal named_rules
            named_rules = self._rule_factory.get_rules(named_rules)
            self.register_message_handler(coro, named_rules + list(rules))
            return coro

        return decorator

    def register_event_handler(
        self, coro: typing.Callable, event_type: Event, rules: typing.List
    ):
        """
        Register an event handler.

        >>> dp.register_event_hanlder(my_handler, Event.WallReplyNew, [])

        :param coro:
        :param event_type:
        :param rules:
        :return:
        """
        handler = self.handler_class(event_type, coro, rules=rules)  # noqa
        self._register_handler(handler)

    def event_handler(self, event_type: Event, *rules, **named_rules):
        """
        Register an event handler with the decorator.

        >>> @dp.event_handler(Event.WALL_REPLY_NEW)
        >>> async def my_func(event: eventobj.WallReplyNew, data: dict):
        >>>     print(event)

        :param event_type:
        :param rules:
        :param named_rules:
        :return:
        """

        def decorator(coro: typing.Callable):
            nonlocal named_rules
            named_rules = self._rule_factory.get_rules(named_rules)
            self.register_event_handler(
                coro, event_type, named_rules + list(rules)
            )
            return coro

        return decorator

    def setup_middleware(self, middleware):
        """
        Append middleware to the middlewares list with middleware manager.
        :param middleware: some middleware
        :return:
        """
        self._middleware_manager.setup(middleware)

    def setup_rule(self, rule):
        """
        Add named rule to named rules list with rule factory.
        :param rule: some rule
        :return:
        """
        self._rule_factory.setup(rule)

    def setup_extension(self, extension: typing.Type[BaseExtension]):
        """
        Add extension to the extensions list with extension manager.
        :param extension: some extension
        :return:
        """
        self._extensions_manager.setup(extension)

    def middleware(self):
        """
        Add middleware to the middlewares list with decorator.
        :return:
        """

        def decorator(middleware_instance: typing.Type[BaseMiddleware]):
            self.setup_middleware(middleware_instance())

        return decorator

    def setup_blueprint(self, blueprint: Blueprint):
        """
        Setup blueprint in application.
        :param blueprint: blueprint instance
        :return:
        """
        for handler in blueprint.handlers.copy():
            named_rules = self._rule_factory.get_rules(handler.named_rules)
            handler.rules.extend(named_rules)
            if handler.event_type is Event.MESSAGE_NEW:
                self.register_message_handler(handler.coro, handler.rules)
            else:
                self.register_event_handler(
                    handler.coro, handler.event_type, handler.rules
                )
        self._registered_blueprints.append(blueprint)

    def run_extension(self, name: str, **extension_init_params):
        """
        Run extensions with the extension manager.
        :param name: name of extension
        :param extension_init_params: params which accept extension constructor
        :return:
        """
        self._extensions_manager.run_extension(name, **extension_init_params)

    @time_logging(logger)
    async def _process_event(self, event: dict):
        """
        Handle 1 event coming from extensions/vk.
        :param event: 1 event coming from extensions/vk
        :return:
        """
        data = (
            {}
        )  # dict to transfer data from middlewares to handlers and filters.
        # examples/bot_framework/simple_middleware.py
        event = get_event_object(event)  # get event pydantic model.

        _skip_handler, data = await self._middleware_manager.trigger_pre_process_middlewares(
            event, data
        )  # trigger pre_process_event funcs in middlewares.
        # return service value '_skip_handler' and data variable (check upper).

        data_.set(data)
        result = False

        logger.debug(f"Pre-process middlewares return this data: {data}")
        logger.debug(
            f"Pre-process middlewares result of skip_handler: {_skip_handler}"
        )

        if (
            not _skip_handler
        ):  # if middlewares don`t skip this handler, dispatcher is gonna check
            # rules and execute handlers.
            for handler in self._handlers:  # check handlers
                if (
                    handler.event_type.value == event.type
                ):  # if handler type is equal event pydantic model.
                    try:
                        if event.type == "message_new":
                            obj = event.object.message
                        else:
                            obj = event.object
                        result = await handler.execute_handler(
                            obj, data
                        )  # if execute hanlder func
                        # return non-False value, other handlers isnt gonna be executed.
                        if result is not False:
                            logger.debug(
                                f"Event handler ({handler.handler.__name__}) successfully executed. Other "
                                f"handlers doesn`t be executed..."
                            )
                            break
                    except Exception:  # noqa
                        logger.exception(
                            f"Error in handler ({handler.handler.__name__}):"
                        )
        if not _skip_handler and result is not False:
            await self._middleware_manager.trigger_post_process_middlewares(
                result
            )
        # trigger post_process_event funcs in middlewares.

    async def _process_events(self, events: typing.List[dict]):
        """
        Process events coming from extensions.
        :param events: list of events coming from extension/vk.
        :return:
        """
        for event in events:
            logger.debug(f"Start processing event with type '{event['type']}'")
            self.vk.loop.create_task(self._process_event(event))

    def run_polling(self, group_id: int = None):
        if not group_id:
            raise TypeError(
                "Group id isn't specified. Use `get_group_id` function for gather it."
            )
        self.run_extension("polling", group_id=group_id, vk=self.vk)



import typing
from enum import Enum

from vk.types.additional import AdminLevel
from vk.types.additional import BlockReason
from vk.types.additional import JoinType
from vk.types.attachments import Photo
from vk.types.attachments.topic import TopicComment
from vk.types.base import BaseModel
from vk.types.client_info import ClientInfo
from vk.types.message import Message
from vk.types.wall_comment import WallComment


class MessageNew(BaseModel):
    message: Message = None
    client_info: ClientInfo = None


class MessageAllow(BaseModel):
    user_id: int = None
    key: typing.Optional[str] = None


class PhotoCommentNew(WallComment):
    photo_id: int = None
    photo_owner_id: int = None


class PhotoCommentDelete(BaseModel):
    owner_id: int = None
    id: int = None
    user_id: int = None
    photo_id: int = None


class VideoCommentNew(WallComment):
    video_id: int = None
    video_owner_id: int = None


class VideoCommentDelete(BaseModel):
    owner_id: int = None
    id: int = None
    user_id: int = None
    video_id: int = None


class WallReplyNew(WallComment):
    post_id: int = None
    post_owner_id: int = None


class WallReplyDelete(BaseModel):
    owner_id: int = None
    id: int = None
    user_id: int = None
    post_id: int = None


class BoardPostNew(TopicComment):
    topic_id: int = None
    topic_owner_id: int = None


class BoardPostDelete(BaseModel):
    topic_id: int = None
    id: int = None


class MarketCommentNew(WallComment):
    market_owner_id: int = None
    item_id: int = None


class MarketCommentDelete(BaseModel):
    owner_id: int = None
    id: int = None
    user_id: int = None
    item_id: int = None


class GroupLeave(BaseModel):
    user_id: int = None
    self: int = None


class GroupJoin(BaseModel):
    user_id: int = None
    join_type: JoinType = None


class UserBlock(BaseModel):
    admin_id: int = None
    user_id: int = None
    unblock_data: int = None
    reason: BlockReason = None
    comment: str = None


class UserUnblock(BaseModel):
    admin_id: int = None
    user_id: int = None
    by_end_date: int = None


class PollVoteNew(BaseModel):
    owner_id: int = None
    poll_id: int = None
    option_id: int = None
    user_id: int = None


class GroupOfficersEdit(BaseModel):
    admin_id: int = None
    user_id: int = None
    level_old: AdminLevel = None
    level_new: AdminLevel = None


class GroupChangeSettingsChangesSectionEnable(Enum):
    status_default = "status_default"
    audio = "audio"
    photo = "photo"
    video = "video"
    market = "market"


class GroupChangeSettingsChangesSectionName(Enum):
    title = "title"
    description = "description"
    community_type = "access"
    screen_name = "screen_name"
    public_category = "public_category"
    public_subcategory = "public_subcategory"
    age_limits = "age_limits"
    website = "website"
    enable_section = GroupChangeSettingsChangesSectionEnable


class GroupChangeSettingsChanges(BaseModel):
    section_name: GroupChangeSettingsChangesSectionName = None
    old_value: typing.Any = None
    new_value: typing.Any = None


class GroupChangeSettings(BaseModel):
    user_id: int = None
    changes: GroupChangeSettingsChanges = None


class GroupChangePhoto(BaseModel):
    user_id: int = None
    photo: Photo = None



import ast

from ..converter import VKScriptConverter


@VKScriptConverter.register(ast.Call)
def call_handler(node: ast.Call):
    converter = VKScriptConverter.get_current()
    replacements = {"append": "push", "pop": "pop", "split": "split"}
    funcs = [
        "slice",
        "push",
        "pop",
        "shift",
        "unshift",
        "splice",
        "substr",
        "split",
        "append",
    ]
    attrs = []
    node_ = node.func
    while isinstance(node_, ast.Attribute):
        attrs.append(node_.attr)
        node_ = node_.value

    if node_.id.upper() == "API" and len(attrs) >= 1:
        if node.args:
            raise TypeError("api calls does not accept positional arguments")

        inner = ",".join(
            f"{keyword.arg}:{converter.convert_node(keyword.value)}"
            for keyword in node.keywords
        )

        return "API." + ".".join(attrs[::-1]) + f"({{{inner}}})"

    elif (
        not attrs
        and node_.id == "len"
        and "len" not in converter.scope.globals
    ):
        return f"{converter.convert_node(node.args[0])}.length"

    # TODO: rewrite?
    elif len(attrs) == 1 and attrs[0] in funcs:
        if node.keywords:
            raise NotImplementedError("keywords not allowed")

        if attrs[0] in replacements:
            func = replacements[attrs[0]]
        else:
            func = attrs[0]

        return (
            f"{converter.convert_node(node_)}.{func}"
            f'({",".join(converter.convert_node(arg) for arg in node.args)})'
        )

    else:
        raise NotImplementedError(
            f'function call "{node_.id + ("." if attrs else "") + ".".join(attrs[::-1])}" not allowed inside VK Script!'
        )



from ..base import BaseModel
from vk.types.attachments import Photo

# https://vk.com/dev/objects/market_album


class MarketAlbum(BaseModel):
    id: int = None
    owner_id: int = None
    title: str = None
    photo: Photo = None
    count: int = None
    updated_time: int = None



from .others import SimpleResponse
from ..base import BaseModel


import typing


class AddResponse(BaseModel):
    likes: int = None


class Add(BaseModel):
    response: AddResponse = None


class Delete(BaseModel):
    response: AddResponse = None


class GetListResponse(BaseModel):
    count: int = None
    items: typing.List[int] = None


class GetList(BaseModel):
    response: GetListResponse = None


class IsLikedResponse(BaseModel):
    liked: int = None
    copied: int = None


class IsLiked(BaseModel):
    response: IsLikedResponse = None



import typing

from ..additional import GeoPlace
from ..base import BaseModel


class GeoCoordinates(BaseModel):
    latitude: typing.Union[int, float] = None
    longitude: typing.Union[int, float] = None


class Geo(BaseModel):
    type: str = None
    coordinates: GeoCoordinates = None
    place: GeoPlace = None



from ..base import BaseModel
from vk.types.attachments import Photo

from typing import Any


class Link(BaseModel):
    url: str = None
    title: str = None
    caption: str = None
    description: str = None
    photo: Photo = None
    is_external: int = None
    product: Any = None
    button: Any = None
    preview_page: str = None
    preview_url: str = None



import logging
import typing
from enum import Enum

from ..exceptions import KeyboardException
from vk.constants import JSON_LIBRARY

logger = logging.getLogger(__name__)


# Keyboards: https://vk.com/dev/bots_docs_3


class ButtonColor(Enum):
    PRIMARY = "primary"  # blue
    SECONDARY = "secondary"  # white
    NEGATIVE = "negative"  # red
    POSITIVE = "positive"  # green


class ButtonType(Enum):
    TEXT = "text"
    LOCATION = "location"
    VKPAY = "vkpay"
    VKAPPS = "open_app"


class Keyboard:
    def __init__(self, one_time: bool, inline: bool = False):
        """
        Create a keyboard object
        :param one_time:
        """
        self.one_time = one_time
        self.buttons = [[]]
        self.keyboard = {
            "one_time": one_time,
            "buttons": self.buttons,
            "inline": inline,
        }

    @staticmethod
    def generate_payload(payload: dict) -> str:
        if payload is None:
            payload = ""
        return payload

    def add_row(self):
        """

        :return:
        """
        if len(self.buttons) >= 10:
            raise KeyboardException("Max 10 rows")

        self.buttons.append([])

    def _add_button(self, action: dict) -> None:
        """

        :param action:
        :return:
        """
        current_row = self.buttons[-1]
        current_row.append(action)

    def add_text_button(
        self,
        text: str,
        color: ButtonColor = ButtonColor.PRIMARY,
        payload: dict = None,
    ):
        """

        :param text:
        :param color:
        :param payload:
        :return:
        """
        payload = self.generate_payload(payload)

        if not isinstance(color, ButtonColor):
            logger.warning("Invalid button color. Used 'PRIMARY'")
            color = ButtonColor.PRIMARY.value
        else:
            color = color.value

        if isinstance(text, str):
            if len(text) < 1:
                raise KeyboardException("Invalid text")
        else:
            raise KeyboardException("Invalid text")

        action = {
            "action": {
                "type": ButtonType.TEXT.value,
                "payload": payload,
                "label": text,
            },
            "color": color,
        }

        self._add_button(action)

    def add_location_button(self, payload: dict = None):
        """

        :param payload:
        :return:
        """

        payload = self.generate_payload(payload)

        action = {
            "action": {"type": ButtonType.LOCATION.value, "payload": payload}
        }

        self._add_button(action)

    def add_vkpay_button(self, hash: str, payload: dict = None):  # noqa
        """
        :param hash:
        :param payload:
        :return:
        """

        payload = self.generate_payload(payload)

        action = {
            "action": {
                "type": ButtonType.VKPAY.value,
                "payload": payload,
                "hash": hash,
            }
        }

        self._add_button(action)

    def add_vkapps_button(
        self, app_id: int, owner_id: int, label: str, payload: dict = None
    ):
        """

        :param app_id:
        :param owner_id:
        :param payload:
        :return:
        """
        payload = self.generate_payload(payload)

        action = {
            "action": {
                "type": ButtonType.VKAPPS.value,
                "app_id": app_id,
                "owner_id": owner_id,
                "payload": payload,
                "label": label,
            }
        }

        self._add_button(action)

    def get_keyboard(self) -> typing.AnyStr:
        """
        Get keyboard json to send.
        If keyboard is 'static', you can generate json once and send it every time.
        :return:
        """
        return JSON_LIBRARY.dumps(self.keyboard)

    @classmethod
    def get_empty_keyboard(cls: "Keyboard") -> typing.AnyStr:
        """

        :return:
        """
        keyboard = cls(one_time=True)  # noqa
        keyboard.keyboard["buttons"] = []
        return keyboard.get_keyboard()



import logging
import re
import typing
from asyncio import iscoroutinefunction

from ..dispatcher import data_
from ..dispatcher.rule import BaseRule
from ..dispatcher.rule import NamedRule
from vk import types
from vk.constants import JSON_LIBRARY
from vk.types.message import Action

try:
    import vbml
except ImportError:
    vbml = None

logger = logging.getLogger(__name__)

"""
Built-in rules.
"""


class Command(BaseRule):
    prefix = "/"
    meta = {
        "name": "Command",
        "description": "A simple rule for checking messages for the specified command",
        "deprecated": False,
    }

    def __init__(self, command: str = None):
        self.command: str = command

    async def check(self, message: types.Message, data: dict):
        msg = message.text.lower()
        result = f"{self.prefix}{self.command}" == msg
        logger.debug(f"Processing text of message. Text in message: {msg}")
        logger.debug(f"Result of Command rule: {result}")
        return result


class Text(NamedRule):
    key = "text"
    IGNORE_CASE = True
    meta = {
        "name": "Text",
        "description": "A simple rule for checking message for the specified text",
        "deprecated": False,
    }

    def __init__(self, text: str):
        self.text: str = text

    async def check(self, message: types.Message, data: dict):
        if self.IGNORE_CASE:
            msg = message.text.lower()
        else:
            msg = message.text
        result = msg == self.text.lower()
        logger.debug(f"Processing text of message. Text in message: {msg}")
        logger.debug(f"Result of Text rule: {result}")
        return result


class Commands(NamedRule):
    key = "commands"

    prefix: typing.Iterable = "/"  # prefixes for command
    IGNORE_CASE = True
    meta = {
        "name": "Commands",
        "description": "A simple rule for checking messages for the specified commands",
        "deprecated": False,
    }

    def __init__(self, commands: typing.List[str]):
        self.commands = commands

    async def check(self, message: types.Message, data: dict):
        passed = False
        if self.IGNORE_CASE:
            msg = message.text.lower().split()[0]
        else:
            msg = message.text.split()[0]
        for command in self.commands:
            if passed:
                break
            for prefix in self.prefix:
                if msg == f"{prefix}{command}":
                    passed = True
                    break

        logger.debug(f"Processing text of message. Text in message: {msg}")
        logger.debug(f"Result of Commands rule: {passed}")
        return passed


class Payload(NamedRule):
    key = "payload"
    meta = {
        "name": "Payload",
        "description": "A simple rule for checking messages for the specified payload",
        "deprecated": False,
    }

    def __init__(self, payload: dict):
        self.payload = payload

    async def check(self, message: types.Message, data: dict):
        if message.payload:
            payload = JSON_LIBRARY.loads(message.payload)
            result = payload == self.payload
            logger.debug(
                f"Processing payload of message. Payload in message: {payload}"
            )
            logger.debug(f"Result of Payload rule: {result}")
            return result


class ChatAction(NamedRule):
    key = "chat_action"
    meta = {
        "name": "ChatAction",
        "description": "A simple rule for checking messages for the specified chat_action",
        "deprecated": False,
    }

    def __init__(self, action: Action):
        self.action = action

    async def check(self, message: types.Message, data: dict):
        if message.action:
            action = Action(message.action.type)
            result = action is self.action
            logger.debug(
                f"Processing action of message. Action in message: {action}"
            )
            logger.debug(f"Result of ChatAction rule: {result}")
            return result


class DataCheck(NamedRule):
    key = "data_check"
    meta = {
        "name": "DataCheck",
        "description": "A simple rule for checking 'data' variable for the specified data",
        "deprecated": False,
    }

    def __init__(self, data: typing.Dict[str, typing.Any]):
        self.data = data  # for example: {"my_key": "my_value"}

    async def check(self, *args):
        data: dict = args[1]
        passed = True
        for key, value in self.data.items():
            value_data = data.get(key)
            if value_data != value:
                passed = False
                break
        logger.debug(f"Result of DataCheck rule: {passed}")
        return passed


class MessageCountArgs(NamedRule):
    """
    Get args and return result of equal len(args) and passed args.
    """

    key = "count_args"
    meta = {
        "name": "MessageCountArgs",
        "description": "A simple rule for checking messages for the specified count of arguments",
        "deprecated": False,
    }

    def __init__(self, count_args: int):
        self.count_args = count_args

    async def check(self, message: types.Message, data: dict):
        count = len(message.get_args())
        result = count == self.count_args
        logger.debug(f"Received {count} args in message")
        logger.debug(f"Result of MessageCountArgs rule: {result}")
        return result


class MessageArgsValidate(NamedRule):
    """
    Get and validate args by passed validators.
    """

    key = "have_args"
    meta = {
        "name": "MessageArgsValidate",
        "description": "A simple rule for checking messages for the specified args",
        "deprecated": False,
    }

    def __init__(
        self,
        args_validators: typing.Union[
            typing.Tuple[int, typing.List[typing.Callable]],
            typing.List[typing.Callable],
        ],
    ):
        if isinstance(args_validators, list):
            self.args_validators = args_validators
            self.delete_element = 1
        elif isinstance(args_validators, tuple):
            if len(args_validators) < 2:
                self.args_validators = args_validators
                self.delete_element = 1
            else:
                self.delete_element = args_validators[0]
                self.args_validators = args_validators[1]

    async def check(self, message: types.Message, data: dict):
        args = message.get_args(self.delete_element)
        count_args = len(args)
        count_validators = len(self.args_validators)
        if count_args != count_validators:
            logger.debug(
                f"Received {count_args} args in message. Passed validators {count_validators}"
            )
            logger.debug(f"Result of MessageArgsValidate rule: False")
            return False
        passed = True
        for validator, arg in zip(self.args_validators, args):
            if iscoroutinefunction(validator):
                result = await validator(arg, message)
            else:
                result = validator(arg)
            if not result:
                logger.debug("Result of MessageArgsValidate rule: False")
                return False
            if isinstance(result, dict):
                data.update(**result)
                data_.set(data)
        logger.debug(f"Result of MessageArgsValidate rule: {passed}")
        if passed:
            return {"args": message.get_args()}
        return passed


class InChat(NamedRule):
    key = "in_chat"
    meta = {
        "name": "InChat",
        "description": "A simple rule for checking messages for the specified peer_id (in chat)",
        "deprecated": False,
    }

    def __init__(self, in_chat: bool):
        self.in_chat: bool = in_chat

    async def check(self, message: types.Message, data: dict):
        result = self.in_chat is bool(message.peer_id >= 2e9)
        logger.debug(f"Received peer_id: {message.peer_id}")
        logger.debug(f"Result of InChat rule: {result}")

        return result


class InPersonalMessages(NamedRule):
    key = "in_pm"
    meta = {
        "name": "InPersonalMessages",
        "description": "A simple rule for checking messages for the specified peer_id (in "
        "personal messages)",
        "deprecated": False,
    }

    def __init__(self, in_pm: bool):
        self.in_pm: bool = in_pm

    async def check(self, message: types.Message, data: dict):
        result = self.in_pm is bool(message.peer_id < 2e9)
        logger.debug(f"Received peer_id: {message.peer_id}")
        logger.debug(f"Result of InPersonalMessages rule: {result}")

        return result


class FromBot(NamedRule):
    key = "from_bot"
    meta = {
        "name": "FromBot",
        "description": "A simple rule for checking messages for the specified peer_id (from bot)",
        "deprecated": False,
    }

    def __init__(self, from_bot: bool):
        self.from_bot: bool = from_bot

    async def check(self, message: types.Message, data: dict):
        result = self.from_bot is bool(message.from_id < 0)
        logger.debug(f"Received from_id: {message.from_id}")
        logger.debug(f"Result of FromBot rule: {result}")

        return result


class WithReplyMessage(NamedRule):
    key = "with_reply_message"
    meta = {
        "name": "WithReplyMessage",
        "description": "A simple rule for checking messages for the availablity reply message",
        "deprecated": False,
    }

    def __init__(self, with_reply_message: bool):
        self.with_reply_message: bool = with_reply_message

    async def check(self, message: types.Message, data: dict):
        logger.debug(
            f"Result of WithReplyMessage rule: {bool(message.reply_message)}"
        )
        return bool(message.reply_message)


class WithFwdMessages(NamedRule):
    key = "with_fwd_messages"
    meta = {
        "name": "WithFwdMessages",
        "description": "A simple rule for checking messages for the availablity forward messages",
        "deprecated": False,
    }

    def __init__(self, with_fwd_messages: bool):
        self.with_reply_message: bool = with_fwd_messages

    async def check(self, message: types.Message, data: dict):
        logger.debug(
            f"Result of WithFwdMessages rule: {bool(message.fwd_messages)}"
        )
        return bool(message.fwd_messages)


class CountFwdMessages(NamedRule):
    key = "count_fwd_messages"
    meta = {
        "name": "CountFwdMessages",
        "description": "A simple rule for checking messages for count of forwarded messages",
        "deprecated": False,
    }

    def __init__(self, count_fwd_messages: int):
        self.count_fwd_messages: int = count_fwd_messages

    async def check(self, message: types.Message, data: dict):
        count = len(message.fwd_messages)
        result = count == self.count_fwd_messages
        logger.debug(f"Received fwd_messages: {count}")
        logger.debug(f"Result of CountFwdMessages rule: {result}")
        return result


class Regex(NamedRule):
    key = "regex"
    meta = {
        "name": "regex",
        "description": "A simple rule for checking message text via regex",
        "deprecated": False,
    }

    def __init__(self, pattern: str):
        self.pattern: typing.Pattern = re.compile(
            pattern, re.IGNORECASE | re.MULTILINE
        )

    async def check(self, message: types.Message, data: dict):
        msg = message.text.lower()
        result = re.search(self.pattern, msg)
        logger.debug(f"Processing text of message. Text in message: {msg}")
        logger.debug(f"Result of Regex rule: {result}")
        return result


class VBML(NamedRule):
    key = "vbml"

    def __init__(self, pattern: typing.Union["vbml.Pattern", str]):
        if isinstance(pattern, str):
            self.pattern = vbml.Patcher.get_current(no_error=False).pattern(
                pattern
            )
        elif isinstance(pattern, vbml.Pattern):
            self.pattern = pattern

        self._patcher = vbml.Patcher.get_current(no_error=False)

    async def check(self, message: types.Message, data: dict):
        result = await self._patcher.check_async(message.text, self.pattern)
        if result is None:
            return
        local_data = {}
        for k, v in result.items():
            local_data[k] = v

        return local_data



from ..base import BaseModel
from vk.types.additional import AudioGenres


# https://vk.com/dev/objects/audio


class Audio(BaseModel):
    id: int = None
    owner_id: int = None
    artist: str = None
    title: str = None
    duration: int = None
    url: str = None
    lyrics_id: int = None
    album_id: int = None
    genre_id: AudioGenres = None
    date: int = None
    no_search: int = 0



"""
A simple cooldown util for message handlers.
"""
import functools
import time

from vk.bot_framework.dispatcher.storage import AbstractAsyncExpiredStorage
from vk.types.message import Message
from vk.utils.mixins import ContextInstanceMixin


class Cooldown(ContextInstanceMixin):
    def __init__(
        self,
        storage: AbstractAsyncExpiredStorage,
        standart_cooldown_time: int = 3,
        for_specify_user: bool = False,
    ):
        self._storage = storage
        self._cooldown_time = standart_cooldown_time
        self._for_specify_user = for_specify_user

        self._cooldown_message: str = "Please, wait: {cooldown} seconds"

    @property
    def cooldown_message(self):
        return self._cooldown_message

    @cooldown_message.setter
    def cooldown_message(self, value: str):
        self._cooldown_message = value

    def cooldown_handler(
        self,
        storage: AbstractAsyncExpiredStorage = None,
        cooldown_time: int = None,
        for_specify_user: bool = None,
        cooldown_message: str = None,
    ):
        """

        :param cooldown_message:
        :param storage:
        :param cooldown_time: standard cooldown time: 3 seconds
        :param for_specify_user: cooldown to specify user
        :return:
        """

        def wrapper(func):
            @functools.wraps(func)
            async def wrapped(*args, **kwargs):
                nonlocal storage, cooldown_time, for_specify_user, cooldown_message
                if storage is None:
                    storage = self._storage
                if cooldown_time is None:
                    cooldown_time = self._cooldown_time
                if for_specify_user is None:
                    for_specify_user = self._for_specify_user
                if cooldown_message is None:
                    cooldown_message = self.cooldown_message

                message: Message = args[0]
                if not isinstance(message, Message):
                    raise RuntimeError(
                        "Cooldown supports only message hanlders"
                    )
                if for_specify_user:
                    cooldown_name = f"__coro_tocooldown:{func.__name__}:user:{message.from_id}_{message.peer_id}__"
                else:
                    cooldown_name = f"__coro_tocooldown:{func.__name__}__"
                have_cooldown = await storage.exists(cooldown_name)
                if have_cooldown:
                    cd = round(
                        (await storage.get(cooldown_name)) - time.time(), 3
                    )
                    answer = cooldown_message.format(cooldown=cd)
                    await message.answer(answer)

                else:
                    result = await func(*args, **kwargs)
                    try:
                        await storage.place(
                            cooldown_name,
                            time.time() + cooldown_time,
                            expire=cooldown_time,
                        )
                    except RuntimeError:
                        pass
                    return result

            return wrapped

        return wrapper



from .base import BaseModel
from .additional import City, Country, Place
from .attachments import MarketPriceCurrency
from enum import IntEnum

import typing

# https://vk.com/dev/objects/group


class CommunityBanInfo(BaseModel):
    end_date: int = None
    comment: str = None


class CommunityContacts(BaseModel):
    user_id: int = None
    desc: str = None
    phone: str = None
    email: str = None


class CommunityCounters(BaseModel):
    photos: int = None
    albums: int = None
    audios: int = None
    videos: int = None
    topics: int = None
    docs: int = None


class CommunityCoverImage(BaseModel):
    url: str = None
    width: int = None
    height: int = None


class CommunityCover(BaseModel):
    enabled: int = None
    images: typing.List[CommunityCoverImage] = None


class CommunityLink(BaseModel):
    id: int = None
    url: str = None
    name: str = None
    desc: str = None
    photo_50: str = None
    photo_100: str = None


class CommunityMainSection(IntEnum):
    no_main_section = 0
    photos = 1
    topics = 2
    audios = 3
    videos = 4
    market = 5


class CommunityMarket(BaseModel):
    enabled: int = None
    price_min: int = None
    price_max: int = None
    main_album_id: int = None
    contact_id: int = None
    currency: MarketPriceCurrency = None
    currency_text: str = None


class CommunityMemberStatus(IntEnum):
    not_a_member = 0
    member = 1
    not_sure = 2
    declined_an_invitation = 3
    sent_request = 4
    invited = 5


class Community(BaseModel):
    id: int = None
    name: str = None
    screen_name: str = None
    is_closed: int = None
    deactivated: str = None
    is_admin: int = None
    admin_level: int = None
    is_member: int = None
    invited_by: int = None
    type: str = None
    has_photo: int = None
    photo_50: str = None
    photo_100: str = None
    photo_200: str = None
    activity: str = None
    age_limits: int = None
    ban_info: CommunityBanInfo = None
    can_create_topic: int = None
    can_message: int = None
    can_post: int = None
    can_see_all_posts: int = None
    can_upload_doc: int = None
    can_upload_video: int = None
    city: City = None
    contacts: CommunityContacts = None
    counters: CommunityCounters = None
    country: Country = None
    cover: CommunityCover = None
    description: str = None
    fixed_post: int = None
    is_favorite: int = None
    is_hidden_from_feed: int = None
    is_messages_blocked: int = None
    links: typing.List[CommunityLink] = None
    main_album_id: int = None
    main_section: CommunityMainSection = None
    market: CommunityMarket = None
    member_status: CommunityMemberStatus = None
    members_count: int = None
    place: Place = None
    public_date_label: str = None
    site: str = None
    start_date: int = None
    finish_date: int = None
    status: str = None
    trending: int = None
    verified: int = None
    wiki_page: str = None



import ast
import functools
import inspect
import types

from vk.utils.vkscript.converter import Scope
from vk.utils.vkscript.converter import VKScriptConverter


# import once
@functools.lru_cache()
def _get_vk() -> "VK":
    from vk import VK

    return VK


def execute(func: types.FunctionType):
    e = Execute()
    return e.decorate(func)


class Execute:
    _code = None
    _preprocessor = None
    _func = None

    def decorate(self, func):
        source = inspect.getsource(func)
        self._func = func
        self._code = ast.parse(source).body[0]
        return self

    def preprocessor(self, func):
        self._preprocessor = func

    def build(self, *args, **kwargs) -> str:
        if self._code.__class__ == ast.FunctionDef:
            globals_ = dict(self._func.__globals__)
            for i, argument in enumerate(self._code.args.args):
                if argument.arg in kwargs:
                    globals_[argument.arg] = kwargs[argument.arg]
                elif i < len(args):
                    globals_[argument.arg] = args[i]
                elif argument.arg.upper() == "API":
                    continue
                else:
                    raise TypeError(
                        f"missing required argument {argument.arg}"
                    )
            converter = VKScriptConverter(Scope(globals=globals_))
            return converter.convert_block(self._code.body)
        raise NotImplementedError()

    async def __call__(self, *args, **kwargs):
        if self._preprocessor is not None:
            return await self._preprocessor(*args, **kwargs)
        return await self.execute(*args, **kwargs)

    async def execute(self, *args, **kwargs):
        vk = _get_vk().get_current()
        code = self.build(*args, **kwargs)
        response = await vk.api_request("execute", {"code": code})
        return response

    e = execute



import logging
import typing
from abc import ABC
from abc import abstractmethod

from vk.utils.mixins import MetaMixin

if typing.TYPE_CHECKING:
    from vk.bot_framework.dispatcher.dispatcher import Dispatcher

T = typing.TypeVar("T")

logger = logging.getLogger(__name__)


class AbstractExtension(ABC, MetaMixin):
    @abstractmethod
    async def get_events(self) -> typing.List:
        """
        Get events from any resource and return list of events.
        :return: list of coming events.
        """

    @abstractmethod
    async def run(self, dp: "Dispatcher"):
        """
        Get events from self.get_events function in the endless of the cycle
        and call dispatcher method dp._process_events.
        :param dp: dispatcher
        :return:
        """


class BaseExtension(AbstractExtension, ABC):
    """
    Can be added to extensions with ExtensionsManager and
    used to get events.

    >>> extension_manager.run_extension(name=unique_key)
    """

    key = None  # unique key to access the extension


class ExtensionsManager:
    def __init__(
        self,
        dp: "Dispatcher",
        default_extensions: typing.Dict[str, typing.Type[BaseExtension]],
    ):
        self.dp: "Dispatcher" = dp
        self.extensions: typing.Dict[str, typing.Type[BaseExtension]] = {}

        self.extensions.update(default_extensions)

    def setup(self, extension: typing.Type[BaseExtension]):
        if extension.key is None:
            raise RuntimeError("Unallowed key for extension")

        self.extensions[extension.key] = extension

    def run_extension(self, name: str, **extension_init_params) -> None:
        """

        :param name: name of the extension
        :param extension_init_params: params which accept the extension constructor
        :return:
        """
        if typing.TYPE_CHECKING:
            BaseExtension = typing.Type[T]  # noqa

        extension: BaseExtension = self.extensions.get(name)  # noqa
        if not extension:
            raise RuntimeError("Undefined extension")

        extension: BaseExtension = extension(**extension_init_params)
        self.dp.vk.loop.create_task(extension.run(self.dp))



import contextvars  # noqa

data_: contextvars.ContextVar[dict] = contextvars.ContextVar(
    "data", default={}
)

from .blueprints import Blueprint
from .dispatcher import Dispatcher
from .extension import BaseExtension
from .handler import Handler
from .storage import AbstractAsyncStorage
from .storage import AbstractStorage
from .storage import Storage



from .base import BaseModel

import typing

# https://vk.com/dev/objects/chat


class ChatPushSettings(BaseModel):
    sound: int = None
    disabled_until: int = None
    disabled_forever: int = None
    no_sound: bool = None


class Chat(BaseModel):
    id: int = None
    type: str = None
    title: str = None
    admin_id: int = None
    users: typing.List[int] = None
    push_settings: ChatPushSettings = None
    photo_50: str = None
    photo_100: str = None
    photo_200: str = None
    left: int = None
    kicked: int = None



from .base import BaseModel
import typing


class Image(BaseModel):
    url: str = None
    width: int = None
    height: int = None


class AppWidgetImage(BaseModel):
    id: str = None
    type: str = None
    images: typing.List[Image] = None



from ..base import BaseModel

# https://vk.com/dev/objects/note


class Note(BaseModel):
    id: int = None
    user_id: int = None
    title: str = None
    text: str = None
    date: int = None
    comments: int = None
    read_comments: int = None



import asyncio
import logging
import typing

from vk import VK
from vk.constants import API_VERSION
from vk.constants import JSON_LIBRARY
from vk.utils import mixins

logger = logging.getLogger(__name__)


# https://vk.com/dev/bots_longpoll


class BotLongPoll(mixins.ContextInstanceMixin):
    def __init__(self, group_id: int, vk: VK):
        """

        :param group_id:
        :param vk:
        """
        self._vk: VK = vk
        self._group_id: int = group_id
        self.server: typing.Optional[str] = None
        self.key: typing.Optional[str] = None
        self.ts: typing.Optional[str] = None

        self.ran = False

    @property
    def vk(self) -> VK:
        return self._vk

    @property
    def group_id(self) -> int:
        return self._group_id

    async def _prepare_longpoll(self):
        await self.vk.api_request(
            "groups.setLongPollSettings",
            {
                "group_id": self.group_id,
                "enabled": 1,
                "api_version": API_VERSION,
            },
        )
        await self._update_polling()

    async def _update_polling(self):
        """
        :return:
        """
        resp = await self.get_server()
        self.server = resp["server"]
        self.key = resp["key"]
        self.ts = resp["ts"]

        logger.debug(
            f"Update polling credentials. Server - {self.server}. Key - {self.key}. TS - {self.ts}"
        )

    async def get_server(self) -> dict:
        """
        Get polling server.
        :return:
        """
        resp = await self.vk.api_request(
            "groups.getLongPollServer", params={"group_id": self.group_id}
        )
        return resp

    async def get_updates(self, key: str, server: str, ts: str) -> dict:
        """
        Get updates from VK.
        :param key:
        :param server:
        :param ts:
        :return:
        """
        async with self.vk.client.post(
            f"{server}?act=a_check&key={key}&ts={ts}&wait=20"
        ) as response:
            resp = await response.json(loads=JSON_LIBRARY.loads)
            logger.debug(f"Response from polling: {resp}")
            return resp

    async def listen(self) -> typing.List[dict]:
        """

        :return: list of updates coming from VK
        """
        try:
            updates: typing.Optional[dict] = await self.get_updates(
                key=self.key, server=self.server, ts=self.ts
            )

            # Handle errors from vkontakte
            if updates.get("failed"):
                logger.debug(
                    f"Longpolling responded with failed: {updates['failed']}"
                )

                if updates["failed"] == 1:
                    self.ts: str = updates["ts"]
                elif updates["failed"] in (2, 3):
                    await self._update_polling()

                return []

            if "ts" not in updates or "updates" not in updates:
                raise Exception("Vkontakte responded with incorrect response")

            self.ts: str = updates["ts"]

            logger.debug(f"Got updates through polling: {updates['updates']}")

            return updates["updates"]

        except Exception:  # noqa
            logger.exception(
                "Received exception while polling... Sleeping 10 seconds..."
            )

            await asyncio.sleep(10)
            await self._update_polling()

            return []

    async def run(self) -> dict:
        """

        :return: last update coming from VK
        """

        await self._prepare_longpoll()
        self.ran = True
        logger.info("Polling started!")

        while True:
            events = await self.listen()
            while events:
                event = events.pop()
                yield event



from .others import SimpleResponse
from ..base import BaseModel


import typing


class CheckPhone(SimpleResponse):
    pass


class RestoreResponse(BaseModel):
    success: int = None
    sid: typing.Any = None



from .messages import Messages
from .account import Account
from .apps import Apps
from .appwidgets import AppWidgets
from .auth import Auth
from .board import Board
from .database import Database
from .docs import Docs
from .fave import Fave
from .friends import Friends
from .gifts import Gifts
from .groups import Groups
from .leads import Leads
from .likes import Likes
from .market import Market

from .api import API



import pydantic

from vk.utils.mixins import ContextInstanceMixin


class BaseModel(pydantic.BaseModel, ContextInstanceMixin):
    class Config:
        allow_mutation = False
        use_enum_values = True

    def __init__(self, *args, **kwargs):
        super(BaseModel, self).__init__(*args, **kwargs)
        self.set_current(self)

    def __str__(self):
        return str(self.dict(skip_defaults=True))

    def __repr__(self):
        args = ", ".join(
            [
                f"{key}={value}"
                for key, value in self.dict(skip_defaults=True).items()
            ]
        )
        return "{}({})".format(self.__class__.__name__, args)

    @property
    def vk(self):
        from vk import VK

        vk = VK.get_current()
        if vk is None:
            raise RuntimeError(
                "Can't get VK instance from context. "
                "You can fix it with setting current instance: "
                "'VK.set_current(vk_instance)'"
            )
        return vk

    @property
    def api(self):
        from vk.methods import API

        api = API.get_current()
        if api is None:
            raise RuntimeError(
                "Can't get API instance from context. "
                "You can fix it with setting current instance: "
                "'API.set_current(API_instance)'"
            )
        return api



import functools
from timeit import default_timer as timer


def time_logging(logger):
    def wrapper(func):
        @functools.wraps(func)
        async def wrapped(*args, **kwargs):
            timer_start = timer()
            logger.debug(
                f"Start processing coroutine ({func.__qualname__})..."
            )
            result = await func(*args, **kwargs)
            time_result = timer() - timer_start
            logger.debug(
                f"Coroutine ({func.__qualname__}) proccessed. Took {time_result:.3f} seconds."
            )
            return result

        return wrapped

    return wrapper



from ..base import BaseModel
from vk.types.additional import PhotoSizes

import typing

# https://vk.com/dev/objects/photo


class Photo(BaseModel):
    id: int = None
    album_id: int = None
    owner_id: int = None
    user_id: int = None
    text: str = None
    date: int = None
    sizes: typing.List[PhotoSizes] = []
    width: int = None
    height: int = None
    access_key: str = None


class PostedPhoto(BaseModel):
    id: int = None
    owner_id: int = None
    photo_130: str = None
    photo_604: str = None


class CropPhotoCrop(BaseModel):
    x: int = None
    y: int = None
    x2: int = None
    y2: int = None


class CropPhoto(BaseModel):
    photo: Photo = None
    crop: CropPhotoCrop = None
    rect: CropPhotoCrop = None



from .others import SimpleResponse
from ..base import BaseModel
from ..application import Application

from ..user import User


import typing


class DeleteAppRequests(SimpleResponse):
    pass


class GetResponse(BaseModel):
    count: int = None
    items: typing.List[Application] = None


class Get(BaseModel):
    response: GetResponse = None


class GetCatalog(Get):
    pass


class GetFriendsListResponse(BaseModel):
    count: int = None
    items: typing.List[User] = None


class GetFriendsList(BaseModel):
    response: GetFriendsListResponse = None


class GetLeaderboard(BaseModel):
    response: typing.List[User] = None


class GetScopesResponse(BaseModel):
    count: int = None
    items: typing.List[dict] = None


class GetScopes(BaseModel):
    response: GetScopesResponse = None


class GetScore(BaseModel):
    response: typing.Any = None


class SendRequest(SimpleResponse):
    pass



from .base import BaseMethod
from vk.types.responses import docs as m


class Docs(BaseMethod):
    async def add(
        self, owner_id: int = None, doc_id: int = None, access_key: str = None
    ):
        """
        Copy a document to a user's or community's document list.
        :param owner_id: ID of the user or community that owns the document. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        :param access_key: Access key. This parameter is required if 'access_key' was returned with the document's data.


        """
        method = self.get_method_name(self.add)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Add(**r)

    async def delete(self, owner_id: int = None, doc_id: int = None):
        """
        Delete a user or community document.
        :param owner_id: ID of the user or community that owns the document. Use a negative value to designate a community ID.
        :param doc_id: Document ID.


        """
        method = self.get_method_name(self.delete)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Delete(**r)

    async def edit(
        self,
        owner_id: int = None,
        doc_id: int = None,
        title: str = None,
        tags: list = None,
    ):
        """
        Edit a document.
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        :param title: Document title.
        :param tags: Document tags.


        """
        method = self.get_method_name(self.edit)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Edit(**r)

    async def get(
        self,
        count: int = None,
        offset: int = None,
        type: int = None,
        owner_id: int = None,
    ):
        """
        Return detailed information about user or community documents.
        :param count: Number of documents to return. By default, all documents.
        :param offset: Offset needed to return a specific subset of documents.
        :param type:
        :param owner_id: ID of the user or community that owns the documents. Use a negative value to designate a community ID.


        """
        method = self.get_method_name(self.get)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Get(**r)

    async def get_by_id(self, docs: list = None):
        """
        Return information about documents by their IDs.
        :param docs: Document IDs. Example: , "66748_91488,66748_91455",


        """
        method = self.get_method_name(self.get_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetById(**r)

    async def get_messages_upload_server(
        self, type: str = None, peer_id: int = None
    ):
        """
        Return the server address for document upload.
        :param type: Document type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "


        """
        method = self.get_method_name(self.get_messages_upload_server)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetMessagesUploadServer(**r)

    async def get_types(self, owner_id: int = None):
        """
        Return documents types available for current user.
        :param owner_id: ID of the user or community that owns the documents. Use a negative value to designate a community ID.


        """
        method = self.get_method_name(self.get_types)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetTypes(**r)

    async def get_upload_server(self, group_id: int = None):
        """
        Return the server address for document upload.
        :param group_id: Community ID (if the document will be uploaded to the community).


        """
        method = self.get_method_name(self.get_upload_server)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetUploadServer(**r)

    async def get_wall_upload_server(self, group_id: int = None):
        """
        Return the server address for document upload onto a user's or community's wall.
        :param group_id: Community ID (if the document will be uploaded to the community).


        """
        method = self.get_method_name(self.get_wall_upload_server)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetWallUploadServer(**r)

    async def save(
        self, file: str = None, title: str = None, tags: str = None
    ):
        """
        Save a document after [vk.com/dev/upload_files_2|uploading it to a server].
        :param file: This parameter is returned when the file is [vk.com/dev/upload_files_2|uploaded to the server].
        :param title: Document title.
        :param tags: Document tags.


        """
        method = self.get_method_name(self.save)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Save(**r)

    async def search(
        self,
        q: str = None,
        search_own: bool = None,
        count: int = None,
        offset: int = None,
    ):
        """
        Return a list of documents matching the search criteria.
        :param q: Search query string.
        :param search_own:
        :param count: Number of results to return.
        :param offset: Offset needed to return a specific subset of results.


        """
        method = self.get_method_name(self.search)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Search(**r)



from . import additional
from . import attachments
from .base import BaseModel
from .community import Community
from .events.community.event import BaseEvent
from .events.community.event import MessageNew
from .events.community.events_list import Event as BotEvent
from .message import Action
from .message import Message
from .user import User



from typing import List

from ..base import BaseModel


class AudioMsg(BaseModel):
    id: int = None
    owner_id: int = None
    duration: int = None
    waveform: List[int] = None
    link_ogg: str = None
    link_mp3: str = None
    access_key: str = None



from typing import Union

from .base import BaseMethod
from vk.types.responses import messages as m


class Messages(BaseMethod):
    async def add_chat_user(self, chat_id: int = None, user_id: int = None):
        """
        Add a new user to a chat.
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be added to the chat.


        """
        method = self.get_method_name(self.add_chat_user)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AddChatUser(**r)

    async def allow_messages_from_group(
        self, group_id: int = None, key: str = None
    ):
        """
        Allow sending messages from community to the current user.
        :param group_id: Group ID.
        :param key:


        """
        method = self.get_method_name(self.allow_messages_from_group)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.AllowMessagesFromGroup(**r)

    async def create_chat(self, user_ids: list = None, title: str = None):
        """
        Create a chat with several participants.
        :param user_ids: IDs of the users to be added to the chat.
        :param title: Chat title.


        """
        method = self.get_method_name(self.create_chat)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.CreateChat(**r)

    async def delete(
        self,
        message_ids: list = None,
        spam: bool = None,
        group_id: int = None,
        delete_for_all: bool = None,
    ):
        """
        Delete one or more messages.
        :param message_ids: Message IDs.
        :param spam: '1' — to mark message as spam.
        :param group_id: Group ID (for group messages with user access token)
        :param delete_for_all: '1' — delete message for for all.


        """
        method = self.get_method_name(self.delete)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Delete(**r)

    async def delete_chat_photo(
        self, chat_id: int = None, group_id: int = None
    ):
        """
        Delete a chat's cover picture.
        :param chat_id: Chat ID.
        :param group_id:


        """
        method = self.get_method_name(self.delete_chat_photo)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.DeleteChatPhoto(**r)

    async def delete_conversation(
        self, user_id: int = None, peer_id: int = None, group_id: int = None
    ):
        """
        Delete all private messages in a conversation.
        :param user_id: User ID. To clear a chat history use 'chat_id'
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with user access token)


        """
        method = self.get_method_name(self.delete_conversation)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.DeleteConversation(**r)

    async def deny_messages_from_group(self, group_id: int = None):
        """
        Deny to send message from community to the current user.
        :param group_id: Group ID.


        """
        method = self.get_method_name(self.deny_messages_from_group)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.DenyMessagesFromGroup(**r)

    async def edit(
        self,
        peer_id: int = None,
        message: str = None,
        message_id: int = None,
        lat: Union[int, float] = None,
        long: Union[int, float] = None,
        attachment: str = None,
        keep_forward_messages: bool = None,
        keep_snippets: bool = None,
        group_id: int = None,
        dont_parse_links: bool = None,
    ):
        """
        Edit the message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param message_id:
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param keep_forward_messages: '1' — to keep forwarded, messages.
        :param keep_snippets: '1' — to keep attached snippets.
        :param group_id: Group ID (for group messages with user access token)
        :param dont_parse_links:


        """
        method = self.get_method_name(self.edit)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Edit(**r)

    async def edit_chat(self, chat_id: int = None, title: str = None):
        """
        Edit the title of a chat.
        :param chat_id: Chat ID.
        :param title: New title of the chat.


        """
        method = self.get_method_name(self.edit_chat)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.EditChat(**r)

    async def get_by_conversation_message_id(
        self,
        peer_id: int = None,
        conversation_message_ids: list = None,
        extended: bool = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return messages by their IDs within the conversation.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param conversation_message_ids: Conversation message IDs.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.get_by_conversation_message_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetByConversationMessageId(**r)

    async def get_by_id(
        self,
        message_ids: list = None,
        preview_length: int = None,
        extended: bool = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return messages by their IDs.
        :param message_ids: Message IDs.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.get_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetById(**r)

    async def get_chat_preview(
        self, peer_id: int = None, link: str = None, fields: list = None
    ):
        """

        :param peer_id:
        :param link: Invitation link.
        :param fields: Profile fields to return.


        """
        method = self.get_method_name(self.get_chat_preview)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetChatPreview(**r)

    async def get_conversation_members(
        self, peer_id: int = None, fields: list = None, group_id: int = None
    ):
        """
        Return a list of IDs of users participating in a chat.
        :param peer_id: Peer ID.
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.get_conversation_members)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetConversationMembers(**r)

    async def get_conversations(
        self,
        offset: int = None,
        count: int = None,
        filter: str = None,
        extended: bool = None,
        start_message_id: int = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return a list of the current user's conversations.
        :param offset: Offset needed to return a specific subset of conversations.
        :param count: Number of conversations to return.
        :param filter: Filter to apply: 'all' — all conversations, 'unread' — conversations with unread messages, 'important' — conversations, marked as important (only for community messages), 'unanswered' — conversations, marked as unanswered (only for community messages)
        :param extended: '1' — return extra information about users and communities
        :param start_message_id: ID of the message from what to return dialogs.
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.get_conversations)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetConversations(**r)

    async def get_conversations_by_id(
        self,
        peer_ids: list = None,
        extended: bool = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return conversations by their IDs
        :param peer_ids: Destination IDs. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param extended: Return extended properties
        :param fields: Profile and communities fields to return.
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.get_conversations_by_id)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetConversationsById(**r)

    async def get_history(
        self,
        offset: int = None,
        count: int = None,
        user_id: int = None,
        peer_id: int = None,
        start_message_id: int = None,
        rev: int = None,
        extended: bool = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return message history for the specified user or group chat.
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param user_id: ID of the user whose message history you want to return.
        :param peer_id:
        :param start_message_id: Starting message ID from which to return history.
        :param rev: Sort order: '1' — return messages in chronological order. '0' — return messages in reverse chronological order.
        :param extended: Information whether the response should be extended
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.get_history)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetHistory(**r)

    async def get_history_attachments(
        self,
        peer_id: int = None,
        media_type: str = None,
        start_from: str = None,
        count: int = None,
        photo_sizes: bool = None,
        fields: list = None,
        group_id: int = None,
        preserve_order: bool = None,
        max_forwards_level: int = None,
    ):
        """
        Return media files from the dialog or group chat.
        :param peer_id: Peer ID. ", For group chat: '2000000000 + chat ID' , , For community: '-community ID'"
        :param media_type: Type of media files to return: *'photo',, *'video',, *'audio',, *'doc',, *'link'.,*'market'.,*'wall'.,*'share'
        :param start_from: Message ID to start return results from.
        :param count: Number of objects to return.
        :param photo_sizes: '1' — to return photo sizes in a
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param group_id: Group ID (for group messages with group access token)
        :param preserve_order:
        :param max_forwards_level:


        """
        method = self.get_method_name(self.get_history_attachments)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetHistoryAttachments(**r)

    async def get_invite_link(
        self, peer_id: int = None, reset: bool = None, group_id: int = None
    ):
        """

        :param peer_id: Destination ID.
        :param reset: 1 — to generate new link (revoke previous), 0 — to return previous link.
        :param group_id: Group ID


        """
        method = self.get_method_name(self.get_invite_link)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetInviteLink(**r)

    async def get_last_activity(self, user_id: int = None):
        """
        Return a user's current status and date of last activity.
        :param user_id: User ID.


        """
        method = self.get_method_name(self.get_last_activity)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetLastActivity(**r)

    async def get_long_poll_history(
        self,
        ts: int = None,
        pts: int = None,
        preview_length: int = None,
        onlines: bool = None,
        fields: list = None,
        events_limit: int = None,
        msgs_limit: int = None,
        max_msg_id: int = None,
        group_id: int = None,
        lp_version: int = None,
        last_n: int = None,
        credentials: bool = None,
    ):
        """
        Return updates in user's private messages.
        :param ts: Last value of the 'ts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param pts: Lsat value of 'pts' parameter returned from the Long Poll server or by using [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param onlines: '1' — to return history with online users only.
        :param fields: Additional profile [vk.com/dev/fields|fields] to return.
        :param events_limit: Maximum number of events to return.
        :param msgs_limit: Maximum number of messages to return.
        :param max_msg_id: Maximum ID of the message among existing ones in the local copy. Both messages received with API methods (for example, , ), and data received from a Long Poll server (events with code 4) are taken into account.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version:
        :param last_n:
        :param credentials:


        """
        method = self.get_method_name(self.get_long_poll_history)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetLongPollHistory(**r)

    async def get_long_poll_server(
        self,
        need_pts: bool = None,
        group_id: int = None,
        lp_version: int = None,
    ):
        """
        Return data required for connection to a Long Poll server.
        :param need_pts: '1' — to return the 'pts' field, needed for the [vk.com/dev/messages.getLongPollHistory|messages.getLongPollHistory] method.
        :param group_id: Group ID (for group messages with user access token)
        :param lp_version: Long poll version


        """
        method = self.get_method_name(self.get_long_poll_server)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.GetLongPollServer(**r)

    async def is_messages_from_group_allowed(
        self, group_id: int = None, user_id: int = None
    ):
        """
        Return information whether sending messages from the community to current user is allowed.
        :param group_id: Group ID.
        :param user_id: User ID.


        """
        method = self.get_method_name(self.is_messages_from_group_allowed)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.IsMessagesFromGroupAllowed(**r)

    async def join_chat_by_invite_link(self, link: str = None):
        """

        :param link: Invitation link.


        """
        method = self.get_method_name(self.join_chat_by_invite_link)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.JoinChatByInviteLink(**r)

    async def mark_as_answered_conversation(
        self, peer_id: int = None, answered: bool = None, group_id: int = None
    ):
        """
        Mark and unmarks conversations as unanswered.
        :param peer_id: ID of conversation to mark as important.
        :param answered: '1' — to mark as answered, '0' — to remove the mark
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.mark_as_answered_conversation)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.MarkAsAnsweredConversation(**r)

    async def mark_as_important(
        self, message_ids: list = None, important: int = None
    ):
        """
        Mark and unmarks messages as important (starred).
        :param message_ids: IDs of messages to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star


        """
        method = self.get_method_name(self.mark_as_important)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.MarkAsImportant(**r)

    async def mark_as_important_conversation(
        self, peer_id: int = None, important: bool = None, group_id: int = None
    ):
        """
        Mark and unmarks conversations as important.
        :param peer_id: ID of conversation to mark as important.
        :param important: '1' — to add a star (mark as important), '0' — to remove the star
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.mark_as_important_conversation)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.MarkAsImportantConversation(**r)

    async def mark_as_read(
        self,
        message_ids: list = None,
        peer_id: int = None,
        start_message_id: int = None,
        group_id: int = None,
    ):
        """
        Mark messages as read.
        :param message_ids: IDs of messages to mark as read.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param start_message_id: Message ID to start from.
        :param group_id: Group ID (for group messages with user access token)


        """
        method = self.get_method_name(self.mark_as_read)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.MarkAsRead(**r)

    async def pin(self, peer_id: int = None, message_id: int = None):
        """
        Pin a message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param message_id:


        """
        method = self.get_method_name(self.pin)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Pin(**r)

    async def remove_chat_user(
        self, chat_id: int = None, user_id: int = None, member_id: int = None
    ):
        """
        Allow the current user to leave a chat or, if the current user started the chat, allows the user to remove another user from the chat.
        :param chat_id: Chat ID.
        :param user_id: ID of the user to be removed from the chat.
        :param member_id:


        """
        method = self.get_method_name(self.remove_chat_user)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.RemoveChatUser(**r)

    async def restore(self, message_id: int = None, group_id: int = None):
        """
        Restore a deleted message.
        :param message_id: ID of a previously-deleted message to restore.
        :param group_id: Group ID (for group messages with user access token)


        """
        method = self.get_method_name(self.restore)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Restore(**r)

    async def search(
        self,
        q: str = None,
        peer_id: int = None,
        date: int = None,
        preview_length: int = None,
        offset: int = None,
        count: int = None,
        extended: bool = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return a list of the current user's private messages that match search criteria.
        :param q: Search query string.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param date: Date to search message before in Unixtime.
        :param preview_length: Number of characters after which to truncate a previewed message. To preview the full message, specify '0'. "NOTE: Messages are not truncated by default. Messages are truncated by words."
        :param offset: Offset needed to return a specific subset of messages.
        :param count: Number of messages to return.
        :param extended:
        :param fields:
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.search)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Search(**r)

    async def search_conversations(
        self,
        q: str = None,
        count: int = None,
        extended: bool = None,
        fields: list = None,
        group_id: int = None,
    ):
        """
        Return a list of the current user's conversations that match search criteria.
        :param q: Search query string.
        :param count: Maximum number of results.
        :param extended: '1' — return extra information about users and communities
        :param fields: Profile fields to return.
        :param group_id: Group ID (for group messages with user access token)


        """
        method = self.get_method_name(self.search_conversations)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SearchConversations(**r)

    async def send(
        self,
        user_id: int = None,
        random_id: int = None,
        peer_id: int = None,
        domain: str = None,
        chat_id: int = None,
        user_ids: list = None,
        message: str = None,
        lat: Union[int, float] = None,
        long: Union[int, float] = None,
        attachment: str = None,
        reply_to: int = None,
        forward_messages: list = None,
        forward: str = None,
        sticker_id: int = None,
        group_id: int = None,
        keyboard: str = None,
        payload: str = None,
        dont_parse_links: bool = None,
        disable_mentions: bool = None,
    ):
        """
        Send a message.
        :param user_id: User ID (by default — current user).
        :param random_id: Unique identifier to avoid resending the message.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param domain: User's short address (for example, 'illarionov').
        :param chat_id: ID of conversation the message will relate to.
        :param user_ids: IDs of message recipients (if new conversation shall be started).
        :param message: (Required if 'attachments' is not set.) Text of the message.
        :param lat: Geographical latitude of a check-in, in degrees (from -90 to 90).
        :param long: Geographical longitude of a check-in, in degrees (from -180 to 180).
        :param attachment: (Required if 'message' is not set.) List of objects attached to the message, separated by commas, in the following format: "<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, 'wall' — wall post, '<owner_id>' — ID of the media attachment owner. '<media_id>' — media attachment ID. Example: "photo100172_166443618"
        :param reply_to:
        :param forward_messages: ID of forwarded messages, separated with a comma. Listed messages of the sender will be shown in the message body at the recipient's. Example: "123,431,544"
        :param forward:
        :param sticker_id: Sticker id.
        :param group_id: Group ID (for group messages with group access token)
        :param keyboard:
        :param payload:
        :param dont_parse_links:
        :param disable_mentions:


        """
        method = self.get_method_name(self.send)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Send(**r)

    async def set_activity(
        self,
        user_id: int = None,
        type: str = None,
        peer_id: int = None,
        group_id: int = None,
    ):
        """
        Change the status of a user as typing in a conversation.
        :param user_id: User ID.
        :param type: 'typing' — user has started to type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'chat_id', e.g. '2000000001'. For community: '- community ID', e.g. '-12345'. "
        :param group_id: Group ID (for group messages with group access token)


        """
        method = self.get_method_name(self.set_activity)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetActivity(**r)

    async def set_chat_photo(self, file: str = None):
        """
        Set a previously-uploaded picture as the cover picture of a chat.
        :param file: Upload URL from the 'response' field returned by the [vk.com/dev/photos.getChatUploadServer|photos.getChatUploadServer] method upon successfully uploading an image.


        """
        method = self.get_method_name(self.set_chat_photo)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.SetChatPhoto(**r)

    async def unpin(self, peer_id: int = None, group_id: int = None):
        """

        :param peer_id:
        :param group_id:


        """
        method = self.get_method_name(self.unpin)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Unpin(**r)



import ast
import string

from ..converter import VKScriptConverter


@VKScriptConverter.register(ast.Expr)
def expr_handler(node: ast.Expr):
    converter = VKScriptConverter.get_current()
    return converter.convert_node(node.value) + ";"


@VKScriptConverter.register(ast.Module)
def module_handler(node: ast.Module):
    converter = VKScriptConverter.get_current()
    return converter.convert_block(node.body)


@VKScriptConverter.register(ast.BinOp)
def bin_op_handler(node: ast.BinOp):
    ops = {
        ast.Add: "+",
        ast.Sub: "-",
        ast.Mult: "*",
        ast.Div: "/",
        ast.Pow: "**",
        ast.RShift: ">>",
        ast.LShift: "<<",
        ast.BitOr: "|",
        ast.BitAnd: "&",
        ast.Mod: "%",
    }

    if node.op.__class__ not in ops:
        raise NotImplementedError(f"Operation {node.op} is not implemented.")
    converter = VKScriptConverter.get_current()
    return f"{converter.convert_node(node.left)}{ops[node.op.__class__]}{converter.convert_node(node.right)}"


@VKScriptConverter.register(ast.Compare)
def compare_handler(node: ast.Compare):
    ops = {
        ast.Gt: ">",
        ast.Lt: "<",
        ast.GtE: ">=",
        ast.LtE: "<=",
        ast.Eq: "==",
        ast.NotEq: "!=",
    }

    operations = []
    converter = VKScriptConverter.get_current()
    left = converter.convert_node(node.left)
    for op, comparator in zip(node.ops, node.comparators):
        if op.__class__ not in ops:
            raise NotImplementedError(
                f"comparison operator {op} not supported"
            )
        operations.append(
            f"{left}{ops[op.__class__]}{converter.convert_node(comparator)}"
        )
    return "&&".join(operations)


@VKScriptConverter.register(ast.BoolOp)
def bool_op_handler(node: ast.BoolOp):
    ops = {ast.And: "&&", ast.Or: "||"}
    if node.op.__class__ not in ops:
        raise NotImplementedError(f"operation '{node.op}' not supported")
    converter = VKScriptConverter.get_current()
    return ops[node.op.__class__].join(
        converter.convert_node(value) for value in node.values
    )


@VKScriptConverter.register(ast.UnaryOp)
def unary_op_handler(node: ast.UnaryOp):
    ops = {ast.UAdd: "+", ast.USub: "-"}
    if node.op.__class__ not in ops:
        raise NotImplementedError(f"operation '{node.op}' not supported")
    converter = VKScriptConverter.get_current()
    return f"{ops[node.op.__class__]}{converter.convert_node(node.operand)}"


@VKScriptConverter.register(ast.Subscript)
def subscript_handler(node: ast.Subscript):
    converter = VKScriptConverter.get_current()
    value = converter.convert_node(node.value)
    if node.slice.__class__ == ast.Index:
        safe = frozenset(string.ascii_letters + string.digits + "_")
        if (
            node.slice.value.__class__ == ast.Str
            and set(node.slice.value.s) <= safe
        ):
            # TODO: Improve safety check, first symbol may be digit
            return f"{value}.{node.slice.value.s}"
        return f"{value}[{converter.convert_node(node.slice.value)}]"
    elif node.slice.__class__ == ast.Slice:
        if node.slice.step:
            raise NotImplementedError("steps in slice not supported")
        if (
            node.slice.lower.__class__ != ast.Num
            and node.slice.upper.__class__ != ast.Num
        ):
            raise TypeError("slices must be integers")
        lower = node.slice.lower.n or 0
        if node.slice.upper:
            return f"{value}.slice({lower},{node.slice.upper.n})"
        return f"{value}.slice({lower})"
    raise NotImplementedError(f"slice {node.slice} not supported")


@VKScriptConverter.register(ast.Attribute)
def attribute_handler(node: ast.Attribute):
    converter = VKScriptConverter.get_current()
    return f"{converter.convert_node(node.value)}.{node.attr}"



from .base import BaseMethod
from vk.types.responses import appwidgets as m


class AppWidgets(BaseMethod):
    async def update(self, code: str = None, type: str = None):
        """
        Allow to update community app widget
        :param code:
        :param type:


        """
        method = self.get_method_name(self.update)
        params = self.create_params(locals())
        r = await self.api_request(method, params)
        return m.Update(**r)



import typing
from typing import List

from ..base import BaseModel
from vk.types.additional import PhotoSizes
from vk.types.attachments import Photo


# https://vk.com/dev/objects/poll


class PollAnswer(BaseModel):
    id: int = None
    text: str = None
    votes: int = None
    rate: typing.Union[int, float] = None


class PollBackgroundPoint(BaseModel):
    position: typing.Union[int, float] = None
    color: str = None


class PollBackground(BaseModel):
    id: int = None
    type: str = None
    angle: int = None
    color: str = None
    width: int = None
    height: int = None
    images: List[PhotoSizes] = None
    points: List[PollBackgroundPoint]


class PollFriends(BaseModel):
    id: int = None


class Poll(BaseModel):
    id: int = None
    owner_id: int = None
    created: int = None
    question: str = None
    votes: int = None
    answers: List[PollAnswer] = None
    anonymous: bool = None
    multiple: bool = None
    answer_ids: List[int] = None
    end_date: int = None
    closed: bool = None
    is_board: bool = None
    can_edit: bool = None
    can_vote: bool = None
    can_report: bool = None
    can_share: bool = None
    author_id: int = None
    photo: Photo = None
    background: PollBackground = None
    friends: List[PollFriends] = None



from .others import SimpleResponse
from ..base import BaseModel

from ..additional import Place
from ..community import Community
from ..user import User

import typing


class AddAddress(BaseModel):
    response: Place = None


class AddCallbackServerResponse(BaseModel):
    server_id: int = None


class AddCallbackServer(BaseModel):
    response: AddCallbackServerResponse = None


class AddLinkResponse(BaseModel):
    id: int = None
    url: str = None
    edit_title: int = None
    name: str = None
    desc: str = None
    image_processing: int = None


class AddLink(BaseModel):
    response: AddLinkResponse = None


class ApproveRequest(SimpleResponse):
    pass


class Ban(SimpleResponse):
    pass


class Create(BaseModel):
    response: Community = None


class DeleteAddress(SimpleResponse):
    pass


class DeleteCallbackServer(SimpleResponse):
    pass


class DeleteLink(SimpleResponse):
    pass


class DisableOnline(SimpleResponse):
    pass


class Edit(SimpleResponse):
    pass


class EditAddress(BaseModel):
    response: Place = None


class EditCallbackServer(SimpleResponse):
    pass


class EditLink(SimpleResponse):
    pass


class EditManager(SimpleResponse):
    pass


class EnableOnline(SimpleResponse):
    pass


class GetResponse(BaseModel):
    count: int = None
    items: typing.List[Community] = None


class Get(BaseModel):
    response: GetResponse = None


class GetAddressesResponse(BaseModel):
    count: int = None
    items: typing.List[Place] = None


class GetAdresses(BaseModel):
    response: GetAddressesResponse = None


class GetBannedResponse(BaseModel):
    count: int = None
    items: typing.List = None


class GetBanned(BaseModel):
    response: GetBannedResponse = None


class GetById(BaseModel):
    response: typing.List[Community] = None


class GetCallbackConfirmationCodeResponse(BaseModel):
    code: str = None


class GetCallbackConfirmationCode(BaseModel):
    response: GetCallbackConfirmationCodeResponse = None


class GetCallbackServersItem(BaseModel):
    id: int = None
    title: str = None
    creator_id: int = None
    url: str = None
    secret_key: str = None
    status: str = None


class GetCallbackServersResponse(BaseModel):
    count: int = None
    items: typing.List[GetCallbackServersItem] = None


class GetCallbackServers(BaseModel):
    response: GetCallbackServersResponse = None


class GetCallbackSettingsResponse(BaseModel):
    api_version: str = None
    events: typing.Dict[str, int] = None


class GetCallbackSettings(BaseModel):
    response: GetCallbackSettingsResponse = None


class GetCatalogResponse(BaseModel):
    count: int = None
    items: typing.List[Community] = None


class GetCatalog(BaseModel):
    response: GetCatalogResponse = None


class GetCatalogInfoResponse(BaseModel):
    enabled: int = None
    id: int = None
    name: str = None
    subcategories: list = None
    page_count: int = None
    page_privews: list = None


class GetCatalogInfo(BaseModel):
    response: GetCatalogInfoResponse = None


class GetInvitedUsersResponse(BaseModel):
    count: int = None
    items: typing.List[User] = None


class GetInvitedUsers(BaseModel):
    response: GetInvitedUsersResponse = None


class GetInvitesResponse(BaseModel):
    count: int = None
    items: typing.List[Community] = None


class GetInvites(BaseModel):
    response: GetInvitesResponse = None


class GetLongPollServerResponse(BaseModel):
    key: str = None
    server: str = None
    ts: int = None


class GetLongPollServer(BaseModel):
    response: GetLongPollServerResponse = None


class GetLongPollSettingsResponse(BaseModel):
    events: typing.Dict[str, int] = None
    is_enabled: bool = None
    api_version: str = None


class GetLongPollSettings(BaseModel):
    response: GetLongPollSettingsResponse = None


class GetMembersResponse(BaseModel):
    count: int = None
    items: typing.List[int] = None


class GetMembers(BaseModel):
    response: GetMembersResponse = None


class GetOnlineStatusResponse(BaseModel):
    status: str = None
    minutes: int = None


class GetOnlineStatus(BaseModel):
    response: GetOnlineStatusResponse = None


class GetRequestsResponse(BaseModel):
    count: int = None
    items: typing.List[User] = None


class GetRequests(BaseModel):
    response: GetRequestsResponse = None


class GetSettings(BaseModel):
    response: typing.Any = None


class GetTokenPermissionsSetting(BaseModel):
    setting: int = None
    name: str = None


class GetTokenPermissionsResponse(BaseModel):
    mask: int = None
    settings: typing.List[GetTokenPermissionsSetting] = None


class Invite(SimpleResponse):
    pass


class IsMemberResponse(BaseModel):
    member: int = None
    can_invite: int = None


class IsMember(BaseModel):
    response: IsMemberResponse = None


class Join(SimpleResponse):
    pass


class Leave(SimpleResponse):
    pass


class RemoveUser(SimpleResponse):
    pass


class ReorderLink(SimpleResponse):
    pass


class SearchResponse(BaseModel):
    count: int = None
    items: typing.List[Community] = None


class Search(BaseModel):
    response: SearchResponse = None


class SetCallbackSettings(SimpleResponse):
    pass


class SetLongPollSettings(SimpleResponse):
    pass


class SetSettings(SimpleResponse):
    pass


class Unban(SimpleResponse):
    pass



from .others import SimpleResponse
from ..base import BaseModel

from ..attachments.topic import TopicComment, Topic
from ..user import User

import typing


class AddTopic(SimpleResponse):
    pass


class CloseTopic(SimpleResponse):
    pass


class CreateComment(SimpleResponse):
    pass


class DeleteComment(SimpleResponse):
    pass


class DeleteTopic(SimpleResponse):
    pass


class EditComment(SimpleResponse):
    pass


class EditTopic(SimpleResponse):
    pass


class FixTopic(SimpleResponse):
    pass


class GetCommentsResponse(BaseModel):
    count: int = None
    items: typing.List[TopicComment] = None
    profiles: typing.List[User] = None


class GetComments(BaseModel):
    response: GetCommentsResponse = None


class GetTopicsResponse(BaseModel):
    count: int = None
    items: typing.List[Topic] = None
    default_order: int = None
    can_add_topics: int = None
    profiles: typing.List[User] = None


class GetTopics(BaseModel):
    response: GetTopicsResponse = None


class OpenTopic(SimpleResponse):
    pass


class RestoreComment(SimpleResponse):
    pass


class UnfixTopic(SimpleResponse):
    pass



from typing import TYPE_CHECKING

from vk.methods import Account
from vk.methods import Apps
from vk.methods import AppWidgets
from vk.methods import Auth
from vk.methods import Board
from vk.methods import Database
from vk.methods import Docs
from vk.methods import Fave
from vk.methods import Friends
from vk.methods import Gifts
from vk.methods import Groups
from vk.methods import Leads
from vk.methods import Likes
from vk.methods import Market
from vk.methods import Messages
from vk.utils import ContextInstanceMixin

if TYPE_CHECKING:
    from vk import VK


class API(ContextInstanceMixin):
    def __init__(self, vk: "VK"):
        self.vk = vk

        self.account = Account(vk, category="account")
        self.messages = Messages(vk, category="messages")
        self.apps = Apps(vk, category="apps")
        self.appwidgets = AppWidgets(vk, category="appwidgets")
        self.auth = Auth(vk, category="auth")
        self.board = Board(vk, category="board")
        self.database = Database(vk, category="database")
        self.docs = Docs(vk, category="docs")
        self.fave = Fave(vk, category="fave")
        self.friends = Friends(vk, category="friends")
        self.gifts = Gifts(vk, category="gifts")
        self.groups = Groups(vk, category="groups")
        self.leads = Leads(vk, category="leads")
        self.likes = Likes(vk, category="likes")
        self.market = Market(vk, category="market")



import ast

from ..converter import VKScriptConverter


@VKScriptConverter.register(ast.Return)
def return_handler(node: ast.Return):
    converter = VKScriptConverter.get_current()
    if node.value is None:
        return "return null;"
    return f"return {converter.convert_node(node.value)};"


@VKScriptConverter.register(ast.Delete)
def delete_handler(node: ast.Attribute):
    converter = VKScriptConverter.get_current()
    return "".join(
        f"delete {converter.convert_node(target)};" for target in node.targets
    )


@VKScriptConverter.register(ast.Pass)
def pass_handler(node: ast.Pass):
    return ""
